#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		gm.tcl (sort gainmatch initialisation tool)	#
#									#
#	Author:		Janet Sampson					#
#									#
#	Last revision:	1/4/97						#
#									#
#########################################################################

# To do, 1/4/97
# 1. Fix custom pattern match... applied even if custom not selected.
# 2. Autogain.... need v/c?

set debug 0

set frame GM
set frametitle "Sort Package: Gainmatch Setup"
set icontitle "Gainmatch"
set started 0

set specserver 0
set specservernames {/hist direct}
set specservername ""

set specdir ""

set selectopt 0
set selectoptnames {
   *.4MeV.q0
   *.20MeV.q0
   Ge*.4MeV.q0
   Clover*.4MeV.q0
   Cluster*.4MeV.q0
   custom
   }
set patternmatch ""

set source 0
#set sourcenames {Eu152 Co60 Co56}
set sourcenames {Eu152 Co60}

set outputfile "$env(HOME)/"

set details 0
set selection {}
set filelist {}
set dirlist {}

set calfits {linear quadratic}
set calfit ""
set cal 0

# Co60 parameters
set co60(0) 1173.0
set co60(1) 1332.0

# Co56 parameters
set co56(0)  846.812
set co56(1) 1037.879
set co56(2) 1238.317
set co56(3) 1771.51
set co56(4) 2034.96
set co56(5) 2598.57
set co56(6) 3253.60

# Eu152 parameters
set eu(0)  121.78
set eu(1)  244.69
set eu(2)  344.27
set eu(3)  778.90
set eu(4)  964.131
set eu(5) 1112.12
set eu(6) 1408.01
##set euI(0) 1362.0; set euI(1) 358.0; set euI(2) 1275.0; set euI(3) 619.0
##set euI(4) 692.0; set euI(5) 649.0; set euI(6) 1000.0
set peak(C) 0.0; set peak(deltaC) 0.0
set peak(E) 0.0; set peak(deltaE) 0.0
set peak(BA) 0.0; set peak(PA) 0.0; set peak(W) 0.0
set coef(A) 0.0; set coef(B) 0.0; set coef(C) 0.0; set coef(D) 0.0

set seg(none)    0
set seg(A)       0
set seg(B)       1
set seg(C)       2
set seg(D)       3
set seg(E)       4
set seg(F)       5
set seg(G)       6

set segletters {A B C D E F G}

proc make-frame {} {
   global env
   global specservernames selectoptnames sourcenames calfits
   set gmx 180
   set gmw 37
   :frame [self frame-name] :label "[self frame-title]" :show-footer T \
         :icon-image "@$env(ICONHOME)/sort" \
         :icon-name "[self icon-title]"
# :toplevel T
      :panel P6 :layout B
         :galley GALLEY :rows 16 :cols 100
         :message M1  :bold T  :label "E R R O R    M E S S A G E S"
      :panel P5 :layout B :padx 30 :pady 5 :fill X
         :layout L
         :button MYHELP       :label Help    :bold T
         :button RUN        :label Run     :bold T
         :button REFRESH    :label Refresh :bold T
      :panel P1 :layout L :fill NONE :padx 5 :pady 7
         :layout T
         :choice-stack SPECSERVER :label "Spectrum server:" :valuex $gmx \
               :strings $specservernames :value 0 :bold T
         :text SPECDIR :label "Spectrum directory:" :w $gmw :valuex $gmx :bold T
         :choice-stack SELECTOPT :label "Selection pattern:" :valuex $gmx \
               :strings $selectoptnames :value 0 :bold T
         :text SELECTSTRING :label "Custom pattern match:" :w 20 :valuex $gmx \
               :bold T
         :choice-stack SOURCE :label "Calibration source:" :valuex $gmx \
               :strings $sourcenames :value 0 :bold T
         :choice-stack CALIBRATION :label "Calibration type:" :valuex $gmx \
               :strings $calfits :value 0 :bold T
         :text OUTPUTFILE :label "Gainmatch output file:" :w $gmw \
               :valuex $gmx :bold T
      :panel P4 :layout L :fill X :padx 5
         :layout T
         :message M2 :label "gainmatch spectra" :bold T
         :list SPECLIST :rows 14 :cols 30 :deaf T
   :show T
   }

#------------------------------------------------------------------------------

proc insert-log s {
   send sl insert [transparent $s]
   }

proc quote-list l {
   set ll ""
   foreach f $l {append ll " " "'$f'"}
   return $ll
   }

proc init-gm-frame {} {
   global debug
   global errorCode
   global started
   global env
   global specserver specservernames specservername
   global patternmatch selectopt selectoptnames
   global source sourcenames sourcename
   global cal calfits calfit
   if {$debug} then {puts stderr "init-gm-frame"}
   set started 1

# display window
   make-frame
#   :modify SPECSERVER :strings $specservernames :value 0
#   :modify SELECTOPT :strings $selectoptnames :value 0
#   :modify SOURCE :strings $sourcenames :value 0
#   :modify CALIBRATION :strings $calfits :value 0
   :modify GALLEY :empty T :inactive T

   set z [catch {error "set errorCode"} m]

# initialise widget values
#   set specserver 0
   set specservername [lindex $specservernames $specserver]

#   set selectopt 0
   if {$selectopt < 7} then {
      set patternmatch [lindex $selectoptnames $selectopt]
      }

#   set source 0
   set sourcename [lindex $sourcenames $source]

   set calfit [lindex $calfits $cal]

   do-repaint
   }

proc bring-to-front f {
   global debug
   if {$debug} then {puts stderr "bring-to-front: $f"}
   wm deiconify $f
   raise $f
   }

#------------------------------------------------------------------------------
proc click-specserver v {
   global debug
   global specserver specservernames specservername
   if {$debug} {puts stderr "click-specserver: '$v'"}
   clear-footer
   set specserver $v
   set specservername [lindex $specservernames $specserver]
   do-repaint
   }

proc click-specdir s {
   global debug
   global specdir
   if {$debug} {puts stderr "click-specdir: '$s'"}
   clear-footer
   set specdir $s
   do-repaint
   }

proc click-source v {
   global source sourcenames sourcename
   global cal calfits calfit
   clear-footer
   set source $v
   set sourcename [lindex $sourcenames $source]
   :modify SOURCE :value $source
   if {$sourcename=="Co60"} then {
      set calfit "linear"
      set cal [lsearch $calfits $calfit]
      :modify CALIBRATION :value $cal
      }
   }

proc click-calibration t {
   global cal calfits calfit
   global sourcename
   clear-footer
   set cal $t
   set calfit [lindex $calfits $cal]
   if {$sourcename=="Co60"} then {
      set calfit "linear"
      set cal [lsearch $calfits $calfit]
      }
   :modify CALIBRATION :value $cal
   }

proc click-selectopt v {
   global debug
   global selectopt selectoptnames patternmatch
   if {$debug} {puts stderr "click-selectopt: '$v'"}
   clear-footer
   set selectopt $v
   set patternmatch [lindex $selectoptnames $v]
   if {$debug} {puts stderr "...patternmatch: $patternmatch"}
      :modify SELECTOPT :value $selectopt
   if {$patternmatch != "custom"} {
      :modify SELECTSTRING :value $patternmatch
      } \
   else {
      set patternmatch ""
      :modify SELECTSTRING :value ""
      }
   read-selected-spectra
   paint-selected-spectra
   }

proc click-selectstring s {
   global debug
   global patternmatch
   if {$debug} {puts stderr "click-selectstring: '$s'"}
   clear-footer
   set patternmatch $s
   :modify SELECTSTRING :value $patternmatch
   read-selected-spectra
   paint-selected-spectra
   }

proc click-outputfile s {
   global outputfile
   clear-footer
   set outputfile $s
   :modify OUTPUTFILE :value $outputfile
   }

#------------------------------------------------------------------------------
proc sort-filelist {} {
   global filelist
   set ll {}
   foreach f $filelist {lappend ll [second $f]/[first $f]}
   set ll [lsort $ll]
   set filelist {}
   foreach f $ll {
      set s [split $f "/"]
      lappend filelist [list [second $s] [first $s]]
      }
   }

proc spec-type t {
   if {$t==1} then {return s}
   if {$t==2} then {return d}
   return ?
   }

proc make-details s {
   global debug
   global specservername specdir
   if {$debug} then {puts stderr "make-details: $specservername/$specdir/$s"}
   if {$specservername == "direct"} {
       set specpath /$specdir/$s'
   } else {
       set specpath $specservername/$specdir/$s'
   }
   set z [catch {eg "(spectrum :name '$specpath/$s' ?spec)"} m]
   if {$z!=0} then {
      display-eg-error-report self $m "Attempting to read spectrum details" \
                                      $specpath/$s
      clear-footer
      return ""
#      error $m
      }
   set dim [first $m]
   set bases [second $m]
   set ranges [third $m]
   set ll ""
   if {$dim>=1} then {append ll "[first $bases]:[first $ranges]"}
   if {$dim>=2} then {append ll "x[second $bases]:[second $ranges]"}
   if {$dim>2} then {append ll "x..."}
   return $ll
   }

proc read-directory {} {
   global debug
   global specservername specdir filelist
   if {$debug} then {puts stderr "read-directory: $specservername/$specdir"}
   if {$specservername == "direct"} {
       set specpath /$specdir
   } else {
       set specpath $specservername/$specdir
   }
   set z [catch {eg "(spectrum :path '$specpath' ?names)"} m]
   if {$z!=0} then {
      display-eg-error-report self $m "Attempting to read directory" $specpath
      clear-footer
      return
#      error $m
      }
   set filelist $m
   sort-filelist
   make-dirlist
   }

proc select-matching-files {} {
   global filelist patternmatch selection nselected
   if {$patternmatch==""} then {set selection {}; return}
   set ll {}; set i 0; set nselected 0
   foreach f $filelist {
      set n [second $f]
      if {$n!="." && $n!=".."} then \
         {
         if {[match-name $patternmatch $n]} then {lappend ll $i; incr nselected}
         incr i
         }
      }
   set selection $ll
   }

proc read-selected-spectra {} {
   global debug
   global selectlist selection dirlist
   if {$debug} {puts stderr "read-selected-spectra"}
   read-directory
   select-matching-files
   set selectlist {}
   foreach i $selection {
      lappend selectlist [second [lindex $dirlist $i]]
      }
   }

proc paint-selected-spectra {} {
   global debug
   global selectlist nselected
   if {$debug} {puts stderr "paint-selected-spectra"}
   :modify SPECLIST :empty T :strings $selectlist :see 0
   set-footer "$nselected spectra matching selection pattern"
   }

proc make-selected-spectra {} {
   global dirlist selection specservername specdir
   if {$specservername == "direct"} {
       set specpath $specdir
   } else {
       set specpath $specservername/$specdir
   }
   set ll {}
   if {$specservername!="hist" && $specdir!=""} then {
      foreach i $selection {
         set n [second [lindex $dirlist $i]]
            lappend ll $specpath/$n
         }
      } \
   else {
      foreach i $selection {
         set n [second [lindex $dirlist $i]]
            lappend ll $specservername/$n
         }
      }
   return $ll
   }

proc make-dir-line {t s} {
   global debug
   global details
   if {$debug>2} then {puts stderr "make-dir-line: $t $s"}
   if {$s=="." || $s==".."} then {return ""}
   if {$t==2} then {return [format "%s   %20s" [spec-type $t] $s]}
   if {$t==1} then {
      if {$details} then {return [format "%s   %20s   %s" \
            [spec-type $t] $s [make-details $s]] } \
      else {return [format "%s   %20s" [spec-type $t] $s]}
      }
   return ""
   }

proc make-dirlist {} {
   global debug
   global filelist dirlist global dircount details
   set dirlist {}; set dircount 0
   if {$debug>1} then {puts stderr "make-dirlist: filelist=$filelist"}
   foreach p $filelist {
      set t [first $p]
      set s [second $p]
      set l [make-dir-line $t $s]
      if {$l!=""} then {
         lappend dirlist $l
         incr dircount
         }
      }
   if {$debug>1} then {puts stderr "...dirlist=$dirlist"}
   }

proc get-groupnum {s} {
   global debug
   set number -1
   set name ""
   set name [string range $s [string last "/" $s] end]
   set x [regexp "\[0-9\]+" $name number]
   if {$debug>1} then {puts stderr "group number from $name = $number"}
   return $number
   }

proc get-segment-letter {s} {
# for group<n>.item<n> returns single character representing detector segment
# otherwise error
   global debug
   if {$debug>1} {puts stderr "get-segment-letter: '$s'"}
   set fname ""
# reduce to filename (remove pathname)
   set fname [string range $s [expr [string last "/" $s] + 1] end]
   case [string toupper $fname] in {
     {*.GEA.*}    {return A}
     {*.GEB.*}    {return B}
     {*.GEC.*}    {return C}
     {*.GED.*}    {return D}
     {*.GEE.*}    {return E}
     {*.GEF.*}    {return F}
     {*.GEG.*}    {return G}
     default      {return none}
     }
   }

proc sort-by-number ll {
   global debug
   if {$debug>1} {puts stderr "sort-by-number: $ll"}
   set jj {}
   foreach l $ll {
     set j 0
     foreach y $jj {
       if {$l < $y} then {break}
       incr j
       }
     set jj [linsert $jj $j $l]
     }
   if {$debug>1} {puts stderr "...returning: '$jj'"}
   return $jj
   }

proc abs x {if {$x>=0} then {return $x} else {return [expr 0-$x]}}

proc list-max s {
   set m [first $s]
   foreach i $s {if {$i>$m} then {set m $i}}
   return $m
   }

proc list-min-index s {
   set m [first $s]; set i 0; set j 0
   foreach n $s {if {$n<$m} then {set m $n; set j $i}; incr i}
   return $j
   }
 
proc list-nearest {s v d} {
   set ds {}
   foreach i $s {lappend ds [abs [expr $v-$i]]}
   set i [list-min-index $ds]
   set dd [lindex $ds $i]
   if {$dd>$d} then {error "closest is $i with delta $d"}
   return [lindex $s $i]
   }

proc split-at-equal s {split $s "="}

proc trim s {string trim $s}

proc map {p ss} {
   set ll {}
   foreach s $ss {lappend ll [$p $s]}
   return $ll
   }

proc scan-float s {
   set f 0.0
   scan $s "%f" f
   return $f
   }

proc parse-peak-integrate pp {
   global peak
   foreach p $pp {
      if {[llength $p]<2} then {continue}
      set value 0.0; set value2 0.0
      set label [trim [first $p]]
      set value [scan-float [second $p]]
      set vals [split [second $p] "/"]
      if {[llength $vals]>=2} then \
         {set value2 [scan-float [string trimleft [second $vals] "-"]]}
      if {$label=="Centroid"} then \
         {set peak(C) $value; set peak(deltaC) $value2}
      if {$label=="Energy"} then \
         {set peak(E) $value; set peak(deltaE) $value2}
      if {$label=="Background area"} then {set peak(BA) $value}
      if {$label=="Peak area"} then {set peak(PA) $value}
      if {$label=="Peak width"} then {set peak(W) $value}
      }
   }

proc parse-calibrate pp {
   global coef
   foreach p $pp {
      if {[llength $p]<2} then {continue}
      set value 0.0; set value2 0.0
      set label [trim [first $p]]
      set value [scan-float [second $p]]
      set vals [split [second $p] "/"]
      if {[llength $vals]>=2} then \
         {set value2 [scan-float [string trimleft [second $vals] "-"]]}
      if {$label=="A"} then \
         {set coef(A) $value; set coef(deltaA) $value2}
      if {$label=="B"} then \
         {set coef(B) $value; set coef(deltaB) $value2}
      if {$label=="C"} then \
         {set coef(C) $value; set coef(deltaC) $value2}
      if {$label=="D"} then \
         {set coef(D) $value; set coef(deltaD) $value2}
      }
   }

proc make-calibrate-input {cs deltacs es} {
   global debug
   set lines ""
   for {set i 0} {$i<[llength $cs]} {incr i} {
      append lines "[lindex $cs $i] [lindex $es $i] [lindex $deltacs $i] 0.1 \n"
      }
   if {$debug} then {puts stdout "$lines"}
   return $lines
   }

proc update-calibrations {} {
   }

proc put-errorlog text {
   :modify GALLEY :inactive F
   append text "\n"
   :modify GALLEY :append T :strings $text :inactive T
   }

#==============================================================================

proc confirm {m p} {
   global env debug
   global confirmcontinue
   global frame
    set i [tk_dialog [self frame-name].dialog "Gainmatch" "$m" "warning" 1 "Cancel" "Run Gainmatch"]
    if {$i != 0} then {eval $p}
}

proc click-run {} {
   global debug
   global c_msg
   global outputfile

# check output file either exists and is a file, or doesn't exist at all...
   if {[file exists $outputfile]} then {
      if {![file isfile $outputfile]} then {
         set-footer "Run failed: need valid output filename"
         return
         }
      }

   set c_msg "WARNING!!  This can take several minutes and will freeze all other midas-session windows... are you sure?"
   confirm $c_msg "do-gainmatch-all"
   }

proc do-gainmatch-all {} {
   global debug
   global specdir
   global sourcename calfit outputfile
   global eu co60 peak coef
   global nselected
   global seg

   if {$debug} then {puts "do-gainmatch-all..."}

#------------------------------------------------------------------------------
# this function outputs the STOPPED gain coefficients for each
# spectrum in the list by performing an auto-calibration on it and
# outputting the coefficients in Eurogam sort language format.
#------------------------------------------------------------------------------

   clear-footer
   :modify GALLEY :inactive F
   :modify GALLEY :empty T :inactive T

#------------------------------------------------------------------------------

# open output file...
   set z [catch {open $outputfile w} gainfd]
   if {$z!=0} then {
      display-error-report self "Cannot open output file" $gainfd
      clear-footer
      return
      }
   if {$debug>1} then \
      {puts stderr "opened output file: '$outputfile'..."}
   puts $gainfd "*DATA"
   puts $gainfd "!! gain coefficients calculated by sort gainmatch tool"
   puts $gainfd "!! from $sourcename source spectra for v/c=0.0"

#------------------------------------------------------------------------------

# refresh spectrum list and update selected spectrum list...
   read-selected-spectra
   paint-selected-spectra
   set-footer "Running gainmatch on $nselected spectra..."

#------------------------------------------------------------------------------

# compile list of spectrum names matching specified pattern option...
   set ss [make-selected-spectra]
   if {$debug} {puts stderr "ss = $ss"}

#------------------------------------------------------------------------------

# loop over spectra, save gain coefficients...

   set seg_list(none) {}
   set seg_list(A) {}
   set seg_list(B) {}
   set seg_list(C) {}
   set seg_list(D) {}
   set seg_list(E) {}
   set seg_list(F) {}
   set seg_list(G) {}

#   put-errorlog "GAINMATCH errors:------------------------------------------------------------------------------------"
   put-errorlog "------- Auto-calibration for v/c=0.0: [exec date] -------"
   foreach s $ss {
      set-footer "running gainmatch on $s..."
      update idletasks

      set goto_next_spectrum 0

# derive group number from spectrum name...
      set groupnum [get-groupnum $s]
      if {$groupnum == -1} then {
         put-errorlog "Invalid gainmatch spectrum name: $s"
         put-errorlog "...cannot decode groupnum number"
         put-errorlog ""
         continue
         }

# derive segment letter from spectrum name...
      set z [catch {get-segment-letter $s} segletter]
      if {$z!=0} { continue }
      set segno $seg($segletter)
      if {$debug} then \
         {puts stderr "spectrum '$s': segment '$segletter', no $segno"}

#==============================================================================

# calibrate spectrum according to source option...
      set hw 10
      if {$sourcename == "Eu152"} then {
        set-footer "finding Europium peaks for spectrum $s..."
         set z [catch {exec peaklist  \
            -spectrum $s -limit 1 4095 -peaks 15} m]
         if {$z!=0} then {
            put-errorlog "Obtaining calibration for $s"
            put-errorlog "...$m"
            put-errorlog ""
#            clear-tags
            continue
            }
         if {[llength $m]<11} then {
            put-errorlog "Obtaining calibration for $s"
            put-errorlog "...can't find enough Europium peaks in $s"
            put-errorlog ""
            continue
            }
# the rightmost peak is invariably the 1408keV line
         set c6 [list-max $m]
         set tags {}
         set z [catch {
            foreach i {0 1 2 3 4 5 6} \
               {lappend tags [list-nearest $m [expr $eu($i)/$eu(6)*$c6] 10] }
            } m]
         if {$z!=0} then {
            put-errorlog "Obtaining calibration for $s"
            put-errorlog "...can't locate all 7 Europium peaks---is this an Eu152 spectrum?"
            put-errorlog ""
            continue
            }
#        if {$debug} then {puts stderr "Eu peaks are $tags"}
         } \
      else {if {$sourcename == "Co60"} then {
         set-footer "finding Cobalt 60 peaks for spectrum $s..."
         set z [catch {exec peaklist  \
            -spectrum $s -limit 1 4095 -peaks 2} m]
         if {$z!=0} then {
            put-errorlog "Obtaining calibration for $s"
            put-errorlog "...$m"
            put-errorlog ""
#            clear-tags
            continue
            }
         if {[llength $m]!=2} then {
            put-errorlog "Obtaining calibration for $s"
            put-errorlog "...can't find the two Cobalt peaks"
            put-errorlog ""
            continue
            }
         set tags $m
         } \
      else {if {$sourcename == "Co56"} then {
         set-footer "finding Cobalt 56 peaks for spectrum $s..."
         set z [catch {exec peaklist  \
            -spectrum $s -limit 1 4095 -peaks 15} m]
         if {$z!=0} then {
            put-errorlog "Obtaining calibration for $s"
            put-errorlog "...$m"
            put-errorlog ""
#            clear-tags
            continue
            }
         if {[llength $m]<11} then {
            put-errorlog "Obtaining calibration for $s"
            put-errorlog "...can't find enough Co56 peaks in $s"
            put-errorlog ""
            continue
            }
         set c6 [list-max $m]
         set tags {}
         set z [catch {
            foreach i {0 1 2 3 4 5 6} \
               {lappend tags [list-nearest $m [expr $co56($i)/$co56(6)*$c6] 10] }
            } m]
         if {$z!=0} then {
            put-errorlog "Obtaining calibration for $s"
            put-errorlog "...can't locate all 7 Co56 peaks"
            put-errorlog ""
            continue
            }
#        if {$debug} then {puts stderr "Co56 peaks are $tags"}
         }}}
      set centroids {}
      set deltacentroids {}
      set widths {}
      set peakareas {}
      if {$sourcename == "Eu152"} then {
         foreach i {0 1 2 3 4 5 6} {
            set c [lindex $tags $i]
            set c1 [expr $c-$hw]
            set c2 [expr $c+$hw]
            set-footer "taking $eu($i)keV peak centroid at channel $c..."
            set z [catch {exec integrate  \
               -spectrum $s \
               -limit $c1 $c2} m]
            if {$z!=0} then \
              {
              put-errorlog "Integrating spectrum: $s"
              put-errorlog "...$m"
              put-errorlog ""
              set goto_next_spectrum 1
              break
              }
            parse-peak-integrate [map split-at-equal [split $m ",\n"]]
            lappend centroids $peak(C)
            lappend deltacentroids $peak(deltaC)
            lappend widths $peak(W)
            lappend peakareas $peak(PA)
            }
         if {$goto_next_spectrum} then \
            continue
         } \
      else {if {$sourcename == "Co60"} then {
         foreach i {0 1} {
            set c [lindex $tags $i]
            set c1 [expr $c-$hw]
            set c2 [expr $c+$hw]
            set-footer "taking $co60($i)keV peak centroid at channel $c..."
            set z [catch {exec integrate  \
               -spectrum $s \
               -limit $c1 $c2} m]
            if {$z!=0} then \
              {
              put-errorlog "Integrating spectrum: $s"
              put-errorlog "... $m"
              put-errorlog ""
              set goto_next_spectrum 1
              break
              }
            parse-peak-integrate [map split-at-equal [split $m ",\n"]]
            lappend centroids $peak(C)
            lappend deltacentroids $peak(deltaC)
            lappend widths $peak(W)
            lappend peakareas $peak(PA)
            }
         if {$goto_next_spectrum} then \
            continue
         } \
      else {if {$sourcename == "Co56"} then {
         foreach i {0 1 2 3 4 5 6} {
            set c [lindex $tags $i]
            set c1 [expr $c-$hw]
            set c2 [expr $c+$hw]
            set-footer "taking $co56($i)keV peak centroid at channel $c..."
            set z [catch {exec integrate  \
               -spectrum $s \
               -limit $c1 $c2} m]
            if {$z!=0} then \
              {
              put-errorlog "Integrating spectrum: $s"
              put-errorlog "...$m"
              put-errorlog ""
              set goto_next_spectrum 1
              break
              }
            parse-peak-integrate [map split-at-equal [split $m ",\n"]]
            lappend centroids $peak(C)
            lappend deltacentroids $peak(deltaC)
            lappend widths $peak(W)
            lappend peakareas $peak(PA)
            }
         if {$goto_next_spectrum} then \
            continue
         }}}
      if {$debug>1} then {puts stderr "centroids are $centroids"
         puts stderr "deltacentroids are $deltacentroids"
         puts stderr "widths are $widths"}
   
      set-footer "computing calibration..."
      if {$sourcename == "Eu152"} then {
         set energies [list $eu(0) $eu(1) $eu(2) $eu(3) $eu(4) $eu(5) $eu(6)]
#         set deltacentroids [list 0.1 0.1 0.1 0.1 0.1 0.1]
         set z [catch {exec calibrate -spectrum $s \
            -opt $calfit  -units keV \
           << [make-calibrate-input $centroids $deltacentroids $energies] } m]
         } \
      else {if {$sourcename == "Co60"} then {
         set energies [list $co60(0) $co60(1)]
         set z [catch {exec calibrate -spectrum $s \
            -opt $calfit  -units keV \
            << [make-calibrate-input $centroids $deltacentroids $energies] } m]
         } \
      else {if {$sourcename == "Co56"} then {
         set energies [list $co56(0) $co56(1) $co56(2) $co56(3) $co56(4) $co56(5) $co56(6)]
#         set deltacentroids [list 0.1 0.1 0.1 0.1 0.1 0.1]
         set z [catch {exec calibrate -spectrum $s \
            -opt $calfit  -units keV \
           << [make-calibrate-input $centroids $deltacentroids $energies] } m]
         }}}

      if {$z!=0} then {
         put-errorlog "Deriving calibration for $s"
         put-errorlog "...$m"
         put-errorlog ""
         continue
         }
      insert-log $m
      update-calibrations

      parse-calibrate [map split-at-equal [split $m ",\n"]]
      if {$debug>1} then \
         {puts stderr "coef: A=$coef(A) B=$coef(B) C=$coef(C) D=$coef(D)"}

#==============================================================================

# output stopped coeffs for each group/item combination

      set a [expr 2*$coef(A)]
      set b [expr 2*$coef(B)]
      set c [expr 2*$coef(C)]

      if {$debug} {puts stderr "...groupnum = $groupnum"}
      set segment($segletter,$groupnum) \
                                   [format "%3d (%f %f %f)" $groupnum $a $b $c]
      lappend seg_list($segletter) $groupnum
      }

   if {$debug} then {
      puts stderr "parameters for each segment..."
      foreach segletter {A B C D E F G} \
         {puts stderr "$segletter: [quote-list $seg_list($segletter)]"}
      }

# write GAINARRAY statements and array info
   foreach segletter {A B C D E F G} {
      if {[llength $seg_list($segletter)] > 0} then {
         puts $gainfd "GAINARRAY  seg$segletter       !! segment $segletter"
         set seg_list($segletter) [sort-by-number $seg_list($segletter)]
         foreach groupnum $seg_list($segletter) {
            puts $gainfd [format "%s" $segment($segletter,$groupnum)]
            }
         }
      }

   set segletter none
   if {[llength $seg_list($segletter)] > 0} then {
      puts $gainfd "GAINARRAY  detgains       !! non-segmented detectors"
      set seg_list($segletter) [sort-by-number $seg_list($segletter)]
      foreach groupnum $seg_list($segletter) {
         puts $gainfd [format "%s" $segment($segletter,$groupnum)]
         }
      }

   close $gainfd
   clear-footer
   }
#------------------------------------------------------------------------------

proc null-action {} {
   }

proc click-refresh {} {
   global debug
   global frame
   if {$debug} {puts stderr "click-refresh: $frame"}
   clear-footer
   do-repaint
   }

proc do-repaint {} {
   global specserver specdir selectopt patternmatch
   global source cal outputfile
   :modify SPECSERVER :value $specserver
   :modify SPECDIR :value $specdir
   :modify SELECTOPT :value $selectopt
   :modify SELECTSTRING :value $patternmatch
   :modify SOURCE :value $source
   :modify CALIBRATION :value $cal
   :modify OUTPUTFILE :value $outputfile
   read-selected-spectra
   paint-selected-spectra
   }

proc click-help f {
   send mb goto [transparent "(SX)Gainmatch Setup window"]
   }

proc click-quit {} {
    finish
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}" }
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {remove-eg-error-report self; return}
   if {$c==":quit"} then {click-quit; return}
   case $w in {
      SPECSERVER   {click-specserver $v}
      SPECDIR      {click-specdir $v}
      SOURCE       {click-source $v}
      CALIBRATION  {click-calibration $v}
      SELECTOPT    {click-selectopt $v}
      SELECTSTRING {click-selectstring $v}
      OUTPUTFILE   {click-outputfile $v}
      SPECLIST     {null-action}
      REFRESH      {click-refresh}
      RUN          {click-run}
      MYHELP         {click-help $f}
      default      {puts stdout $arg}
      }
   }

proc start args {
   global started
   if {$started} then {bring-to-front [self frame-name]} \
   else {init-gm-frame}
   }

proc finish args {destroy-frame}
