#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soi-EuroballConfig.tcl		                #
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	9/4/97						#
#									#
#########################################################################

set frametitle "MTsort Euroball Online Sort Input Handler Config"

set currentfamily 0
set currentname   ""
set currentmaxdets  0
set currentbasegroup    0
set currentbasewords  0
set currentauxgroup    0
set currentauxwords  0

set currentmachine ""
set currentport "16000 16100"

set runfileheader "Family   Name                                  MaxDets    Ge(BaseGroup  Nwords)    BGO(BaseGroup  Nwords)"

set z [catch {set env(os)} ostype]
if {$z} {set ostype xxx}
if {$ostype == "SunOS"} {
  set devlist {"Not_Active" "Net" "Other?"}
  } {
  set devlist {"Not_Active" "Net" "Other?"}
  }

#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      ADD                    {click-add}
      REPLACE                {click-replace}
      REMOVE                 {click-remove}
      SAVE                   {click-save}

      SORT-RUN-FRAME         {click-sort-run-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      RUNLIST                {click-runlist $c $v}

      FAMILY                {click-family $v}
      NAME                   {click-name $v}
      MAXDETS             {click-maxdets $v}
      BASEGROUP               {click-basegroup $v}
      BASEWORDS               {click-basewords $v}
      AUXGROUP               {click-auxgroup $v}
      AUXWORDS               {click-auxwords $v}

      INPUT-HANDLER          {click-input-handler $v}

      STREAM                 {click-stream $v}
      MACHINE                {click-machine $v}
      PORT                   {click-port $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug
   global env
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings
   global runfilelist runfileheader

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 12c
  set vx2 19c
  set vx3 300
  set twid1 15
  set twid2 45
  set twid3 30

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button ADD :label "Add"
      :button REPLACE :label "Replace"
      :button REMOVE :label "Remove"
      :button SAVE :label "Save"
      :button SORT-RUN-FRAME :label "Sort Run Frame" :x $vx3
      :button MYHELP :label "Help"
      :button REFRESH :label "Refresh"
   :panel RUNFILES3 :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :text BASEWORDS :label "N words:" :valuex $vx1
      :text AUXWORDS :label "N words:" :valuex $vx1
   :panel RUNFILES2 :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :text BASEGROUP :label "Base Group:" :valuex $vx1
      :text AUXGROUP :label "BGO Group:" :valuex $vx1
   :panel RUNFILES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :message LIST-HEADER :label "$runfileheader" :bold T
     :next-row T
      :list RUNLIST :rows 10 :list-width 500 :x 0 :exclusive T \
             :strings $runfilelist
     :next-row T
      :text FAMILY :label "Family:" :w $twid2 :valuex $vx1
     :next-row T
      :text NAME :label "Name:" :w $twid2 :valuex $vx1
     :next-row T
      :text MAXDETS :label "Max Dets:" :valuex $vx1
  :panel PORTS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10
      :text MACHINE :label "Machine:" :w $twid2 :valuex $vx1
      :message STREAM :label "Stream 1" :bold T
     :next-row T
      :text PORT :label "Port:" :w $twid2 :valuex $vx1
  :panel OPTIONS  :layout L :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 10
      :message INPUT-HANDLER :label "Input handler:    $handlerloaded" :bold T
      :text INPUT-HANDLER-STATUS :label "Status:" :value $status :bold T \
         :w $twid1 :valuex $vx2 :x 300
      [:path INPUT-HANDLER-STATUS].entry configure -bd 0
   :show T
}

proc paint-handler-status {} {
   global status basedir
   set z [mtsort $basedir "i state" status 1]
   :modify INPUT-HANDLER-STATUS :value $status
   }

proc repaint {} {
   global basedir handlerselected handlerloaded
   global devlist device1selected device2selected
   global device2activestates device2active
   global tracelist tracelevel

   check-handler
   :modify INPUT-HANDLER :label "Input handler:    $handlerloaded"
   paint-handler-status

   paint-runfiles
   }

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-midas-error-report self "" $m
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc read-runinfo {} {
    global basedir runfilename runfilelist
    global currentmachine currentport
    set runfilelist {}
    set i 0
    set z 0
    while {$z==0} {
	set i [expr $i + 1]
	set z [read-file $runfilename]
	if {$z!=""} then {
	    foreach l $z {
		if {[second $l] == "port"} {
		    set a [expr [string first "port" $l] + 5]
		    set b [expr [string first "machine" $l] -2]
		    set currentport [string range $l $a $b]
		    set a [expr $b + 10]
		    set currentmachine [string range $l $a end]
		    continue
		}
		if {[first $l] == "machine"} {
		    set currentport "16000 16100"
		    set currentmachine [string range $l 8 end]
		    continue
		}
		set f [format-runfile $l]
		lappend runfilelist $f
	    }
	}
    }
    paint-runfiles
}

proc format-runfile {r} {
   scan $r "%d %s %d %d %d %d %d" s1 s2 i1 i2 i3 i4 i5
   set r [format "%-6s %-16s %8d %8d %8d %8d %8d" $s1 $s2 $i1 $i2 $i3 $i4 $i5]
   return $r
   }

proc paint-runfiles {} {
   global runfilelist runfileselected
   global currentfamily currentname currentmaxdets
   global currentbasegroup currentbasewords
   global currentauxgroup currentauxwords
   global currentmachine currentport
   :modify RUNLIST :empty T :strings $runfilelist :select $runfileselected
   set runfile [lindex $runfilelist $runfileselected]
   set currentfamily [first $runfile]
   set currentname [second $runfile]
   set currentmaxdets [third $runfile]
   set currentbasegroup [fourth $runfile]
   set currentbasewords [fifth $runfile]
   set currentauxgroup [sixth $runfile]
   set currentauxwords [seventh $runfile]
   :modify FAMILY :value $currentfamily
   :modify NAME :value $currentname
   :modify MAXDETS :value $currentmaxdets
   :modify BASEGROUP :value $currentbasegroup
   :modify BASEWORDS :value $currentbasewords
   :modify AUXGROUP :value $currentauxgroup
   :modify AUXWORDS :value $currentauxwords
   :modify MACHINE :value $currentmachine
   :modify PORT :value $currentport
   }
                 
proc click-stream e {
   global basedir
   global device1selected devlist
   frame-busy
   set device1selected $e
   set d [lindex $devlist $e]
    set m ""
   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device $d" m 1]
   if {$z!=0} then {set-footer $m; set device1selected -1} \
   else {
      clear-footer
      }
   repaint
   frame-idle
   }

proc click-machine v {
   global currentmachine
   set currentmachine [string tolower $v]
   }

proc click-port v {
   global currentport
   set currentport [string toupper $v]
   }

proc click-family v {
   global currentfamily
   set currentfamily [string toupper $v]
   }

proc click-name f {
   global currentname
# may not want tolower: may want asis for disc file
   set currentname  [string tolower $f]
   }

proc click-maxdets s {
   global currentmaxdets
   set currentmaxdets  [string toupper $s]
   }

proc click-basegroup e {
   global currentbasegroup
   set currentbasegroup  [string toupper $e]
   }

proc click-basewords e {
   global currentbasewords
   set currentbasewords  [string toupper $e]
   }

proc click-auxgroup e {
   global currentauxgroup
   set currentauxgroup  [string toupper $e]
   }

proc click-auxwords e {
   global currentauxwords
   set currentauxwords  [string toupper $e]
   }

proc click-add {} {
   global runfilelist runfileselected
   global currentfamily currentname currentmaxdets
   global currentbasegroup currentbasewords
   global currentauxgroup currentauxwords
   frame-busy
    if { $currentfamily == ""} return
    if { $currentname == ""} return
    if { $currentmaxdets == ""} return
    if { $currentbasegroup == ""} return
    if { $currentbasewords == ""} return
    if { $currentauxgroup == ""} return
    if { $currentauxwords == ""} return
   set l "$currentfamily $currentname $currentmaxdets $currentbasegroup $currentbasewords $currentauxgroup $currentauxwords"
   lappend runfilelist [format-runfile $l]
   set runfileselected [expr [llength $runfilelist] - 1]
   paint-runfiles
   frame-idle
   }

proc click-replace {} {
   global runfilelist runfileselected
   global currentfamily currentname currentmaxdets
   global currentbasegroup currentbasewords
   global currentauxgroup currentauxwords
   frame-busy
   set l "$currentfamily $currentname $currentmaxdets $currentbasegroup $currentbasewords $currentauxgroup $currentauxwords"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-remove {} {
   global runfilelist runfileselected
   frame-busy
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected]
   set runfileselected [expr $runfileselected - 1]
   if { $runfileselected < 1 } then {set runfileselected 0}
   paint-runfiles
   frame-idle
   }

proc click-save {} {
   global basedir sortdir runfilelist runfilename
   global currentmachine currentport
   frame-busy

    set l "machine $currentmachine"
    set w ""
    lappend w $l
    foreach r $runfilelist {lappend w $r}
    set z [write-file $runfilename $w]
    if {$z!=0} then {
         display-midas-error-report self "" "Error saving runfiles" $m
         frame-idle
         return
         }
   frame-idle
   }

proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   paint-runfiles
   repaint
   frame-idle
   }

proc click-runlist {c f} {
   global debug
   global runfile runfilelist runfileselected
   global currentfamily currentname currentmaxdets
   global currentbasegroup currentbasewords
   global currentauxgroup currentauxwords
   if {$debug} then {puts stderr "click-list: $c $f"}
   clear-footer

   case $c in {
      :list-select {
         set runfile $f
         set currentfamily [first $runfile]
         set currentname [second $runfile]
         set currentmaxdets [third $runfile]
         set currentbasegroup [fourth $runfile]
         set currentbasewords [fifth $runfile]
         set currentauxgroup [sixth $runfile]
         set currentauxwords [seventh $runfile]
         set runfileselected [lsearch $runfilelist $runfile]
        }
     :list-deselect {
        set runfile ""
         set runfileselected 0
        }
     }          
   if {$runfileselected < 1} then { set runfileselected 0 }
   paint-runfiles
   }

proc init-values {} {
   global debug
   global basedir tracelevel tracelist
   global devlist device1selected device2selected
   global runfilename

   read-handler-directory

    set m ""
   set z [mtsort $basedir "i trace ?" m 1]
   set tracelevel $m
   if {$debug} then {puts "soi init: trace $z $m"}

   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device1selected [lsearch $devlist $m] } \
                        else { set device1selected 0 }
   if {$debug} then {puts "soi init: device1 $z $m"}

   set z [mtsort $basedir "i stream 2" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device2selected [lsearch $devlist $m] } \
                        else { set device2selected 0 }
   if {$debug} then {puts "soi init: device2 $z $m"}

   set runfilename $basedir/.mtsort.cf

   return 0
   }


proc start-handler args {
   global started
   if {![init-values]} then { 
      make-frame
      update idletasks
      click-refresh
      set started 1
      }
   }
