#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soi-tape.tcl (sort input tape handler setup)    #
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	26/2/97						#
#									#
#########################################################################

set frametitle "MTsort Input Tape Handler Control"

set z [catch {set env(os)} ostype]
if {$z} {set ostype xxx}
if {$ostype == "SunOS"} {
  set devlist {"Not_Active" "/dev/rmt/0mbn" "/dev/rmt/1mbn" "/dev/rmt/2mbn" "/dev/rmt/0hbn" "/dev/rmt/1hbn" "/dev/rmt/2hbn" "/dev/rmt/0cbn" "/dev/rmt/1lbn" "Other?"}
  } {
  set devlist {"Not_Active" "/dev/nst0" "/dev/nst1" "/dev/nst2" "Other?"}
  }

#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      ADD                    {click-add}
      REPLACE                {click-replace}
      REMOVE                 {click-remove}
      RESETFILE              {click-resetfile}
      RESETALL               {click-resetall}
      SAVE                   {click-save}

      SORT-RUN-FRAME         {click-sort-run-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      RUNLIST                {click-runlist $c $v}

      DEBUG                  {click-debug $v}
      CONFIG                 {click-config $v}
      READRUNFILES           {click-reget-runfiles}

      VOLUME                 {click-volume $v}
      FILE                   {click-file $v}
      STARTBLOCK             {click-startblock $v}
      ENDBLOCK               {click-endblock $v}

      INPUT-HANDLER          {click-input-handler $v}

      STREAM                 {click-stream $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug
   global env
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings
   global runfilelist

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 12c
  set vx2 19c
  set vx3 300
  set vx4 380
  set twid1 15
  set twid2 45

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button ADD :label "Add" :w 2
      :button REPLACE :label "Replace" :w 5
      :button REMOVE :label "Remove" :w 5
      :button RESETALL :label "ResetAll" :w 5
      :button RESETFILE :label "ResetLine" :w 6
      :button SAVE :label "Save" :w 2
      :button SORT-RUN-FRAME :label "Sort Run Frame" :x 350 :w 11
      :button MYHELP :label "Help" :w 3
      :button REFRESH :label "Refresh" :w 5
   :panel RUNFILES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :message LIST-HEADER :label "Input Runfiles" :bold T :x 280
      :list RUNLIST :rows 10 :list-width 500 :x 0 :exclusive T \
             :strings $runfilelist :see 0
     [:path RUNLIST].list configure -font "courier 12"
     :next-row T
      :text VOLUME :label "Volume:" :w $twid2 :valuex $vx1
      :choice-stack DEBUG :label "Debug" :strings $debugmenustrings
     :next-row T
      :text FILE :label "File:" :w $twid2 :valuex $vx1
      :button CONFIG :label "Config" :x $vx4
     :next-row T
      :text STARTBLOCK :label "Start block:" :valuex $vx1
      :button READRUNFILES :label "ReRead from Setup" :x $vx4
     :next-row T
      :text ENDBLOCK :label "End block:" :valuex $vx1
  :panel OPTIONS  :layout L :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 10
      :message INPUT-HANDLER :label "Input handler:    $handlerloaded" :bold T
     :next-row T
      :choice-stack STREAM :label "Stream 1" \
         :valuex $vx1 :strings $devlist :value 0
   :panel INFO  :layout L :padx 10 :pady 10
      :layout V; :row-gap 20; :col-gap 10
      :text INPUT-HANDLER-STATUS :label "Status:" :value $status :bold T \
         :w $twid1 :valuex $vx2
      [:path INPUT-HANDLER-STATUS].entry configure -bd 0
      :text STATISTICS-TOTAL :label "STATISTICS    total:" :value 0 :bold T \
         :w $twid1 :valuex $vx2
      [:path STATISTICS-TOTAL].entry configure -bd 0
      :text STATISTICS-CURRENT :label "current:" :value 0 :bold T \
         :w $twid1 :valuex $vx2
      [:path STATISTICS-CURRENT].entry configure -bd 0
   :show T
}

proc paint-handler-status {} {
   global status basedir
   set z [mtsort $basedir "i state" status 1]
   :modify INPUT-HANDLER-STATUS :value $status
   }

proc repaint {} {
   global basedir handlerselected handlerloaded
   global devlist device1selected device2selected
   global device2activestates device2active
   global tracelist tracelevel

   check-handler
   :modify INPUT-HANDLER :label "Input handler:    $handlerloaded"
   paint-handler-status

   set statistics2 "Unknown q Unknown q"
    set m ""
   if { $device2selected != 0} then {
      set z [mtsort $basedir "i stream 2" m 1]
      set z [mtsort $basedir "i stats" m 1]
      if {$z==0} then { set statistics2 [string range $m 26 end] }
      }
    set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i stats" m 1]
   if {$z} then { set statistics1 "Unknown q Unknown q"} \
   else { set statistics1 [string range $m 26 end] }
   set statistics2 "0 block 0 current"

   if {$device2selected > 0} then {
#      xv "(SOI-STATS-DEV2-VALUE :label '[third $statistics2]')"
#      xv "(SOI-PANEL1A :show F) (SOI-PANEL1B :show T)"
      }

   :modify STATISTICS-TOTAL :value [first $statistics1]
   :modify STATISTICS-CURRENT :value [third $statistics1]
   :modify STREAM :value $device1selected
   :modify DEBUG :value $tracelevel

   paint-runfiles
   }

proc read-runinfo {} {
   global basedir runfilename runfilelist
   set runfilelist {}
   set i 0
   set z 0
    set m ""
   while {$z==0} {
      set i [expr $i + 1]
      set z [mtsort $basedir "i runinfo $i ?" m 1]
      if {$z==0} {
	  if {[string range $m 0 0] != " "} {lappend runfilelist [format-runfile $m]}
      }
   }
   paint-runfiles
   }

proc format-runfile {r} {
   scan $r "%s %s %d %d" s1 s2 i1 i2
   set r [format "%6s %-30s %12d %12d" $s1 $s2 $i1 $i2]
   return $r
   }

proc paint-runfiles {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   :modify RUNLIST :empty T :strings $runfilelist :select $runfileselected :see $runfileselected
    if {$runfilelist != {}} {
   set runfile [lindex $runfilelist $runfileselected]
   set currentvolume [first $runfile]
   set currentfile [second $runfile]
   set currentstart [third $runfile]
   set currentend [fourth $runfile]
   :modify VOLUME :value $currentvolume
   :modify FILE :value $currentfile
   :modify STARTBLOCK :value $currentstart
   :modify ENDBLOCK :value $currentend
   }
}

proc click-stream e {
   global basedir
   global device1selected devlist
   frame-busy
   set device1selected $e
   set d [lindex $devlist $e]
    set m ""
   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device $d" m 1]
   if {$z!=0} then {set-footer $m; set device1selected -1} \
   else {
      set-eg-option mtsort-input-device1 $d
      edit-defaults mtsort-input-device1 "set-eg-option mtsort-input-device1 $d"
      clear-footer
      }
   repaint
   frame-idle
   }

proc click-volume v {
   global currentvolume
   set currentvolume [string toupper $v]
   }

proc no-spaces f {
    set s $f
    string trimleft $s
    while {[set i [string first " " $s]] != -1} {
	set s [string range $s 0 [expr $i -1]][string range $s [expr $i +1] end]
    }

    return $s
}

proc click-file f {
   global currentfile
# may not want tolower: may want asis for disc file
#    set currentfile  [no-spaces [string tolower $f]]
    set currentfile  [no-spaces $f]
   }

proc click-startblock s {
   global currentstart
   set currentstart  [string toupper $s]
   }

proc click-endblock e {
   global currentend
   set currentend  [string toupper $e]
   }

proc click-add {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
   if { $currentvolume == ""} return
   if { $currentfile == ""} return
   if { $currentstart == ""} return
   if { $currentend == ""} return
   set l "$currentvolume $currentfile $currentstart $currentend"
   lappend runfilelist [format-runfile $l]
   set runfileselected [expr [llength $runfilelist] - 1]
   paint-runfiles
   frame-idle
   }

proc click-replace {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
   if {![llength $runfilelist]} then {
      click-add
#      frame-idle
      return
      }
   set l "$currentvolume $currentfile $currentstart $currentend"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-resetfile {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if {![llength $runfilelist]} then {return}
   frame-busy
   set currentstart 0
   set currentend 0
   set l "$currentvolume $currentfile $currentstart $currentend"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-resetall {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if {![llength $runfilelist]} then {return}
   frame-busy

    set currentstart 0
    set currentend 0

    set newlist ""
    foreach runfile $runfilelist {
	set currentvolume [first $runfile]
	set currentfile [second $runfile]
	set l "$currentvolume $currentfile 0 0"
	lappend newlist [format-runfile $l]
    }
   set runfilelist $newlist
   paint-runfiles
   frame-idle
   }

proc click-save {} {
   global basedir sortdir runfilelist
   frame-busy
   set i 0
   set z 0
   set m ""
   foreach x $runfilelist {
      set i [expr $i + 1]
      set z [mtsort $basedir "i runinfo $i $x" m 1]
      if {$z!=0} then {
         display-midas-error-report self "" "Error saving runfiles:\n" $m
         frame-idle
         return
         } \
      else {
         set z [mtsort $basedir "i runinfo $i eof" m 1]
         set-footer $m
         clear-footer
         }
      }
   update-runfiles
   frame-idle
   }

proc click-config e {
   global basedir soap_port
   global device1selected devlist
   inform soic start $soap_port
   }

proc click-runlist {c f} {
   global debug
   global runfile runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if {$debug} then {puts stderr "click-list: $c $f"}
   clear-footer

   case $c in {
      :list-select {
         set runfile $f
         set currentvolume [first $runfile]
         set currentfile [second $runfile]
         set currentstart [third $runfile]
         set currentend [fourth $runfile]
         set runfileselected [lsearch $runfilelist $runfile]
        }
     :list-deselect {
        set runfile ""
         set runfileselected 0
        }
     }          
   if {$runfileselected < 1} then { set runfileselected 0 }
   paint-runfiles
   }

proc init-values {} {
   global debug
   global basedir tracelevel tracelist
   global devlist device1selected device2selected

   read-handler-directory

    set m ""
   set z [mtsort $basedir "i trace ?" m 1]
   set tracelevel $m
   if {$debug} then {puts "soi init: trace $z $m"}

   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device1selected [lsearch $devlist $m] } \
                        else { set device1selected 0 }
   if {$debug} then {puts "soi init: device1 $z $m"}

   set z [mtsort $basedir "i stream 2" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device2selected [lsearch $devlist $m] } \
                        else { set device2selected 0 }
   if {$debug} then {puts "soi init: device2 $z $m"}
   return 0
   }


proc start-handler args {
   global started
   if {![init-values]} then { 
      make-frame
      update idletasks
      click-refresh
      set started 1
      }
   }
