#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soi-XsysFeraConfig.tcl		                #
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	24/09/01					#
#									#
#########################################################################

set frametitle "MTsort XsysFera Sort Input Handler Config"

set runfileheader "Module Name                 VSN  Pattern  Nchans  Ndatabits  Basegroup\nGroup  Number               VSN:SA ..."

set z [catch {set env(os)} ostype]
if {$z} {set ostype xxx}
if {$ostype == "SunOS"} {
  set devlist {"Not_Active" "Net" "Other?"}
  } {
  set devlist {"Not_Active" "Net" "Other?"}
  }

set runfilepreamble ""

#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      ADD                    {click-add}
      INSERT                 {click-insert}
      REPLACE                {click-replace}
      REMOVE                 {click-remove}
      SAVE                   {click-save}

      SORT-RUN-FRAME         {click-sort-run-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      RUNLIST                {click-runlist $c $v}

      AUXWORDS               {click-auxwords $v}

      INPUT-HANDLER          {click-input-handler $v}

      STREAM                 {click-stream $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug
   global env
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings
   global runfilelist runfileheader

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 12c
  set vx2 19c
  set vx3 300
  set twid1 15
  set twid2 60
  set twid3 30

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button ADD :label "Add"
      :button INSERT :label "Insert"
      :button REPLACE :label "Replace"
      :button REMOVE :label "Remove"
      :button SAVE :label "Save"
      :button SORT-RUN-FRAME :label "Sort Run Frame" :x $vx3
      :button MYHELP :label "Help"
      :button REFRESH :label "Refresh"
   :panel RUNFILES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :message LIST-HEADER :label "$runfileheader" :bold T
     :next-row T
      :list RUNLIST :rows 20 :list-width 500 :x 0 :exclusive T \
             :strings $runfilelist
     :next-row T
      :text AUXWORDS :label "Module:" :w $twid2 :valuex $vx1
  :panel OPTIONS  :layout L :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 10
      :message INPUT-HANDLER :label "Input handler:    $handlerloaded" :bold T
      :text INPUT-HANDLER-STATUS :label "Status:" :value $status :bold T \
         :w $twid1 :valuex $vx2 :x 300
      [:path INPUT-HANDLER-STATUS].entry configure -bd 0
     :next-row T
      :message STREAM :label "Stream 1" :bold T
   :show T
}

proc paint-handler-status {} {
   global status basedir
   set z [mtsort $basedir "i state" status 1]
   :modify INPUT-HANDLER-STATUS :value $status
   }

proc repaint {} {
   global basedir handlerselected handlerloaded
   global devlist device1selected device2selected
   global device2activestates device2active
   global tracelist tracelevel

   check-handler
   :modify INPUT-HANDLER :label "Input handler:    $handlerloaded"
   paint-handler-status

   paint-runfiles
   }

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-midas-error-report self "" $m
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    global runfilepreamble
    set runfilepreamble ""
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { [isComment? $t] } { lappend runfilepreamble $t } {
	    lappend buf $t
	}
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    global runfilepreamble
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach l $runfilepreamble {puts $fp $l}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc read-runinfo {} {
   global basedir runfilename runfilelist
   global currentmachine currentport
   set runfilelist {}
   set i 0
   set z 0
   while {$z==0} {
      set i [expr $i + 1]
      set z [read-file $runfilename]
      if {$z!=""} then {
	  foreach l $z {
	      set f [format-runfile $l]
	      lappend runfilelist $f
	  }
      }
   }
   paint-runfiles
   }

proc zero_format_array {} {
    global ii
    foreach n {1 2 3 4 5 6 7 8 9 10 11 12} {set ii($n) ""}
}

proc format-runfile {r} {
    global ii
    if {[first $r] == "module"} {
	scan $r "%s %s %d %d %d %d %d" s1 s2 i1 i2 i3 i4 i5
	set r [format "%-6s %-10s %5d %5d %5d %5d %5d" $s1 $s2 $i1 $i2 $i3 $i4 $i5]
    }
    if {[first $r] == "group"} {
	set s [split $r]
	set n 1
        zero_format_array
	foreach t $s { if {$t != ""} {set ii($n) $t; incr n} }
	set r [format "%-6s %6s %10s %5s %5s %5s %5s %5s %5s %5s" $ii(1) $ii(2) $ii(3) $ii(4) $ii(5) $ii(6) $ii(7) $ii(8) $ii(9) $ii(10) $ii(11) $ii(12)]
    }
   return $r
   }

proc paint-runfiles {} {
    global runfilelist runfileselected
    global runfile
    :modify RUNLIST :empty T :strings $runfilelist :select $runfileselected
    set runfile [lindex $runfilelist $runfileselected]
    if {[string range $runfile 0 0] != "#"} {
	:modify AUXWORDS :value $runfile
    } {
	:modify AUXWORDS :value ""
    }
   }
                 
proc click-stream e {
   global basedir
   global device1selected devlist
   frame-busy
   set device1selected $e
   set d [lindex $devlist $e]
    set m ""
   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device $d" m 1]
   if {$z!=0} then {set-footer $m; set device1selected -1} \
   else {
      clear-footer
      }
   repaint
   frame-idle
   }

proc click-auxwords e {
   global runfile linetype
   set runfile [string tolower $e]
   }

proc click-add {} {
   global runfilelist runfileselected
   global runfile
   frame-busy
    if { $runfile == ""} return
    lappend runfilelist [format-runfile $runfile]
   set runfileselected [expr [llength $runfilelist] - 1]
   paint-runfiles
   frame-idle
   }

proc click-insert {} {
   global runfilelist runfileselected
   global runfile
   frame-busy
    if { $runfile == ""} return
   set l [format-runfile $runfile]
    set runfileselected [incr runfileselected]
   set runfilelist [linsert $runfilelist $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-replace {} {
   global runfilelist runfileselected
   global runfile
   frame-busy
   set l [format-runfile $runfile]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-remove {} {
   global runfilelist runfileselected
   frame-busy
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected]
   set runfileselected [expr $runfileselected - 1]
   if { $runfileselected < 1 } then {set runfileselected 0}
   paint-runfiles
   frame-idle
   }

proc click-save {} {
   global basedir sortdir runfilelist runfilename
   frame-busy

    set w ""
    foreach r $runfilelist {lappend w $r}
    set z [write-file $runfilename $w]
    if {$z!=0} then {
         display-midas-error-report self "" "Error saving runfiles" $m
         frame-idle
         return
         }
   frame-idle
   }

proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   paint-runfiles
   repaint
   frame-idle
   }

proc click-runlist {c f} {
   global debug
   global runfile runfilelist runfileselected
   if {$debug} then {puts stderr "click-list: $c $f"}
   clear-footer

   case $c in {
      :list-select {
         set runfile $f
         set runfileselected [lsearch $runfilelist $f]
        }
     :list-deselect {
        set runfile ""
         set runfileselected 0
        }
     }          
   if {$runfileselected < 1} then { set runfileselected 0 }
   paint-runfiles
   }

proc init-values {} {
   global debug
   global basedir tracelevel tracelist
   global devlist device1selected device2selected
   global runfilename

   read-handler-directory

    set m ""
   set z [mtsort $basedir "i trace ?" m 1]
   set tracelevel $m
   if {$debug} then {puts "soi init: trace $z $m"}

   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device1selected [lsearch $devlist $m] } \
                        else { set device1selected 0 }
   if {$debug} then {puts "soi init: device1 $z $m"}

   set z [mtsort $basedir "i stream 2" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device2selected [lsearch $devlist $m] } \
                        else { set device2selected 0 }
   if {$debug} then {puts "soi init: device2 $z $m"}

   set runfilename $basedir/.mtsort.cf

   return 0
   }

proc click-help {} {
    send help html_show {"" $env(MTSORTBASE)/manual/xsysfera-config.html}
}

# Requires htmlviewer.patch.tcl  to work ...
proc click-help {} {
    global env
    inform htmlviewer startfile $env(MTSORTBASE)/manual/xsysfera-config.html
   }

proc start-handler args {
   global started
   if {![init-values]} then { 
      make-frame
      update idletasks
      click-refresh
      set started 1
      }
   }
