#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soi.tcl (sort input handler setup)		#
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	30/9/97						#
#									#
#########################################################################

set frame SOI
set frametitle "Sort Input Handler Control"
set icontitle "InputHandler"

set debug 0
set started 0
set sync 0

set handlerdir ""
set basedir ""
set sortdir ""
set runfilename ""

set handlerloaded ""
set handlerprefix "MTsortInput"
set handlerselected -1
set handlercurrent -1
set handlersource ""
set handlerlist {}
set handlercount 0
set handler ""

set status "Not Loaded"

set debugmenustrings { Off BlockStats Minimal Events MoreEvents All }
set tracelevel 0

#...???
set device1selected 1
set device2selected 0

set device2activestates {":inactive" "" }
set device2active 0

set runfile ""
set runfileselected -1
set runfilelist {}
set runfilecount 0

set currentvolume ""
set currentfile   ""
set currentstart  0
set currentend    0

set filelist {}

set soap_port 19999
#------------------------------------------------------------------------------

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-midas-error-report self "" $m
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc last p {
   set n [llength $p]
   incr n -1
   return [lindex $p $n]
   }

proc front p {
   set n [llength $p]
   incr n -1
   return [lreplace $p $n $n]
   }

proc paint-handler-status {} {
   global status basedir
   set z [mtsort $basedir "i state" status 1]
   :modify INPUT-HANDLER-STATUS :value $status
   }

proc make-handler-list {} {
   global debug handlerprefix
   global handlerlist handlerselected handlercount filelist
   set handlerlist {}; set handlercount 0
   if {$debug>2} then {puts stderr "make-handlerlist: filelist=$filelist"}
   foreach p $filelist {
      set t [first $p]
      if { [string first $handlerprefix $t]==0} then {
	  set t2 [expr [string first ".so" $t] -1]
         lappend handlerlist [string range $t [string length $handlerprefix] $t2]
         incr handlercount
         }
      }
   set handlerlist [lsort $handlerlist]
   lappend handlerlist ""
   if {$debug>2} then {puts stderr "...handlerlist=$handlerlist"}
   }

proc read-handler-directory {} {
   global handlerdir filelist
   set z [catch {eg "(directory :path '$handlerdir' ?files)"} m]
   if {$z!=0} then {
      display-midas-error-report self $m "Attempting to read directory" $handlerdir
      error $m
      }

   set filelist $m
   make-handler-list
   }

proc check-handler {} {
   global basedir 
   global handlerlist handlercount handlerloaded handlerselected handlerprefix

   read-handler-directory

    set m ""
   set z [mtsort $basedir "i handler ?" m 1]
   if {$z} then {
      set handlerselected -1
      set handlerloaded "No"
      set-footer $m
      } \
   else { 
      set i [string last $handlerprefix $m]
      if {$i != -1} then {
         set handlerloaded [string range $m [expr $i + [string length $handlerprefix]] end]
         set handlerselected [lsearch $handlerlist $handlerloaded]
         }
      }
   }

proc confirm {m action} {
   global debug
   set i [tk_dialog [self frame-name].quitdialog "[self frame-title] Confirmation" $m  "warning" 1 "Cancel" "Confirm"]
   if {$i} then $action
   if {$debug} then {puts stderr "confirm: $m choice $i"}
   }

proc click-handler-go {} {
   global basedir
   frame-busy
   set-footer "Starting handler ..."
    set m ""
   set z [mtsort $basedir "i go" m 1]
   if {$z!=0} then {set-footer $m;} \
   else {
      clear-footer
      }
   repaint
   frame-idle
   }

proc click-handler-stop {} {
   global basedir
   frame-busy
   set-footer "Stopping handler ..."
    set m ""
   set z [mtsort $basedir "i stop" m 1]
   if {$z!=0} then {set-footer $m} \
   else {
      clear-footer
      }
   repaint
   frame-idle
   }

proc click-debug e {
   global basedir tracelevel tracelist
   frame-busy
   set tracelevel $e
    set m ""
   set z [mtsort $basedir "i trace $tracelevel" m 1]
   :modify DEBUG :value $tracelevel
   frame-idle
   }

proc click-sort-run-frame {} {
   clear-footer
   inform soc {start "" ""}
   }

proc click-help {} {
    send help html_show {"" $env(MTSORTBASE)/manual/sortinput.html}
   }

# Requires htmlviewer.patch.tcl  to work ...
proc click-help {} {
    global env
    inform htmlviewer startfile $env(MTSORTBASE)/manual/sortinput.html
   }

proc read-runinfo {} {
   global basedir runfilename runfilelist
   set runfilelist {}
   set i 0
   set z 0
    set m ""
   while {$z==0} {
      set i [expr $i + 1]
      set z [mtsort $basedir "i runinfo $i ?" m 1]
      if {$z==0} then {
         lappend runfilelist [format-runfile $m]
         }
      }
   paint-runfiles
   }

proc click-remove {} {
   global runfilelist runfileselected
   frame-busy
   if {![llength $runfilelist]} then {
      display-midas-error-report self "" "Runfile list is empty"
      frame-idle
      return
      }
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected]
   set runfileselected [expr $runfileselected - 1]
   if { $runfileselected < 1 } then {set runfileselected 0}
   paint-runfiles
   frame-idle
   }

proc update-runfiles {} {
    global basedir runfilelist 

    set r ""
    set fp [fopen $basedir/runinfo r]
    if {$fp != 0} {
	set t [gets $fp]
	if { ! [isComment? $t] } { lappend r $t }
    }
    close $fp

    foreach line $runfilelist {lappend r $line}
    write-file $basedir/runinfo $r
    exec rm $basedir/rundata
    exec csh -c "(cd $basedir;MTrunfiles <runinfo)"
}

proc click-reget-runfiles {} {
    global basedir
    exec rm $basedir/rundata
    exec csh -c "(cd $basedir;MTrunfiles <runinfo)"

   set z [mtsort $basedir "i runfile $basedir/rundata" m 1]
   if {$z} {set-footer $m}
    read-runinfo
}

proc read-runfile {} {
   global debug
   if {$debug} {puts stderr "read-runfile..."}
   send soc call-back soi soi.read-runfile-continue get-sortprogram
   send soc call-back soi soi.read-handlerdir-continue get-handlerdir
   }

proc soi.read-runfile-continue f {
   global debug
   global runfilename runfilelist runfileselected
   global sortdir basedir
   if {$debug}  then {puts stderr "[self frame-name]: call-back result $f"}

   set sortdir $f
   set basedir $sortdir
   set runfilename $sortdir/runinfo

   if {![file exists $runfilename]} then {
      set-footer "Runfile $runfilename does not exist!"
      return
      }
   set runfileselected 0
   }

proc soi.read-handlerdir-continue f {
    global handlerdir
    set handlerdir $f
}

proc load-handler-source {} {
   global handlerloaded handlersource debug

    
    set type ""
    if {[string last "Tape" $handlerloaded] != -1} {set type "Tape"}
    if {[string last "Disc" $handlerloaded] != -1} {set type "Disc"}
    if {[string last "Online" $handlerloaded] != -1} {set type "Online"}
    if {[string last "Remote" $handlerloaded] != -1} {set type "Remote"}
    if {$type == ""} {set type "Tape"}
    set hs soi-$type

   if {$handlerloaded == "EuroballOnline"} {set hs soi-EuroballOnline}
   if {$handlerloaded == "TDROnline"} {set hs soi-Remote}

   if {$handlersource == $hs} {
       click-refresh
       bring-to-front [self frame-name]
       return
   }
   if {$debug} {puts stderr "Loading source for $hs ..."}
   catch {destroy .soi}
   loadsource $hs
   set started 0
   start-handler
   set started 1
   set handlersource $hs
}

proc click-refresh {} {
   global basedir tracelevel
   frame-busy
   clear-footer
   read-runinfo
   paint-runfiles
   repaint
   set z [mtsort $basedir "i trace ?" m 1]
   if { ! $z} {set tracelevel $m}
   :modify DEBUG :value $tracelevel
   frame-idle
   }

proc finish {} {DeleteTask}

proc start args {
   global started handlerselected handlercurrent debug 
   read-runfile
   check-handler
   if {$handlerselected == -1} {return}
   if {$handlerselected != $handlercurrent} {
      load-handler-source
      set handlercurrent $handlerselected
   } {
      if {$started} then {
	  click-refresh
	  bring-to-front [self frame-name]
      }
   }
}

loadsource mtsortcommand
