########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soic.tcl (sort input handler config)		#
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	26/2/97						#
#									#
#########################################################################

set frame SOIC
set frametitle "Sort Input Handler Config"
set icontitle "HandlerConfig"

set debug 0
set started 0
set sync 0

set handlerdir ""
set basedir ""
set sortdir ""
set runfilename ""

set handlerloaded ""
set handlerprefix "MTsortInput"
set handlerselected -1
set handlercurrent -1
set handlersource ""
set handlerlist {}
set handlercount 0
set handler ""

set status "Not Loaded"

set debugmenustrings { Off Minimal More Events MoreEvents All }
set tracelevel 0

#...???
set device1selected 1
set device2selected 0

set device2activestates {":inactive" "" }
set device2active 0

set runfile ""
set runfileselected -1
set runfilelist {}
set runfilecount 0

set currentvolume ""
set currentfile   ""
set currentstart  0
set currentend    0

set filelist {}

#------------------------------------------------------------------------------

proc last p {
   set n [llength $p]
   incr n -1
   return [lindex $p $n]
   }

proc front p {
   set n [llength $p]
   incr n -1
   return [lreplace $p $n $n]
   }

proc confirm {m action} {
   global debug
   set i [tk_dialog [self frame-name].quitdialog "[self frame-title] Confirmation" $m  "warning" 1 "Cancel" "Confirm"]
   if {$i} then $action
   if {$debug} then {puts stderr "confirm: $m choice $i"}
   }


proc click-debug e {
   global basedir tracelevel tracelist
   frame-busy
   set tracelevel $e
    set m ""
   set z [mtsort $basedir "i trace $tracelevel" m 1]
   :modify DEBUG :value $tracelevel
   frame-idle
   }

proc click-sort-run-frame {} {
   clear-footer
   inform soi {start "" ""}
   }

proc click-help {} {
    send help html_show {"" $env(MTSORTBASE)/manual/sortconfig.html}
   }

proc read-runfile {} {
   global debug
   if {$debug} {puts stderr "read-runfile..."}
   send soc call-back soic soic.read-runfile-continue get-sortprogram
   send soc call-back soic soic.read-handlerdir-continue get-handlerdir
   }

proc soic.read-runfile-continue f {
   global debug
   global runfilename runfilelist runfileselected
   global sortdir basedir
   if {$debug}  then {puts stderr "[self frame-name]: call-back result $f"}

   set sortdir $f
   set basedir $sortdir
   set runfilename $sortdir/.mtsort.cf

   if {![file exists $runfilename]} then {
      set-footer "Configfile $runfilename does not exist!"
      return
      }
   set runfileselected 0
   }

proc soic.read-handlerdir-continue f {
    global handlerdir
    set handlerdir $f
}

proc make-handler-list {} {
   global debug handlerprefix
   global handlerlist handlerselected handlercount filelist
   set handlerlist {}; set handlercount 0
   if {$debug>2} then {puts stderr "make-handlerlist: filelist=$filelist"}
   foreach p $filelist {
      set t [first $p]
      if { [string first $handlerprefix $t]==0} then {
	  set t2 [expr [string first ".so" $t] -1]
	  if {[string length $t] != [expr $t2 +4]} continue
         lappend handlerlist [string range $t [string length $handlerprefix] $t2]
         incr handlercount
         }
      }
   set handlerlist [lsort $handlerlist]
   lappend handlerlist ""
   if {$debug>2} then {puts stderr "...handlerlist=$handlerlist"}
   }

proc read-handler-directory {} {
   global handlerdir filelist
   set z [catch {eg "(directory :path '$handlerdir' ?files)"} m]
   if {$z!=0} then {
      display-midas-error-report self $m "Attempting to read directory" $handlerdir
      error $m
      }

   set filelist $m
   make-handler-list
   }

proc check-handler {} {
   global basedir 
   global handlerlist handlercount handlerloaded handlerselected handlerprefix

   read-handler-directory

    set m ""
   set z [mtsort $basedir "i handler ?" m 1]
   if {$z} then {
      set handlerselected -1
      set handlerloaded "No"
      set-footer $m
      } \
   else { 
      set i [string last $handlerprefix $m]
      if {$i != -1} then {
         set handlerloaded [string range $m [expr $i + [string length $handlerprefix]] end]
         set handlerselected [lsearch $handlerlist $handlerloaded]
         }
      }
   }

proc load-handler-source {} {
   global handlerloaded handlersource debug env

    set hs ""
    set hname ""

    if {[string first "Euroball" $handlerloaded] != -1} {set hs soi-EuroballConfig}
    if {[string first "Gasp" $handlerloaded] != -1} {
	if {$hs != ""} {set hs "gaspeuroball"}
    }

    if {[string first "Exogam" $handlerloaded] != -1} {set hs soi-ExogamConfig}

    if {[string first "Xsys" $handlerloaded] != -1} {
	if {[string first "Fera" $handlerloaded] != -1} {
	    set hs soi-XsysFeraConfig
	}
    }

    if {[string first "Tags" $handlerloaded] != -1} {set hs soi-TagsConfig}

    if {[string first "In2p3" $handlerloaded] != -1} {set hs soi-In2p3Config}

    if {[string first "Afrodite" $handlerloaded] != -1} {set hs soi-AfroditeConfig}

# Load generic config handler if config file found ...
    
    if {$hs == ""} {
	set a [string first "Tape" $handlerloaded]
	if { $a != -1} {
	    set hname [string range $handlerloaded 0 [expr $a -1]]
	} {
	    set a [string first "Disc" $handlerloaded]
	    if { $a != -1} {
		set hname [string range $handlerloaded 0 [expr $a -1]]
	    } {
		set a [string first "Online" $handlerloaded]
		if { $a != -1} {
		    set hname [string range $handlerloaded 0 [expr $a -1]]
		} {
		    set a [string first "Remote" $handlerloaded]
		    if { $a != -1} {
			set hname [string range $handlerloaded 0 [expr $a -1]]
		    }
		}
	    }
	}
    }

    if {$hname != ""} {
	set hname [string tolower $hname]
	set cfname "$env(MTSORTBASE)/config/$hname.cf"
	if {[file exists $cfname]} {set hs soi-GenericConfig}
    }

    if {$hs == ""} return

   if {$handlersource == $hs} {
       click-refresh
       bring-to-front [self frame-name]
       return
   }
   if {$debug} {puts stderr "Loading source for $hs ..."}
   catch {destroy .soic}
   loadsource $hs
   set started 0
   start-handler
   set started 1
   set handlersource $hs
}

proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   paint-runfiles
   repaint
   frame-idle
   }

proc finish {} {DeleteTask}

proc start args {
   global started handlerselected handlercurrent debug 
   read-runfile
   check-handler
   if {$handlerselected == -1} {return}
   if {$handlerselected != $handlercurrent} {
      load-handler-source
      set handlercurrent $handlerselected
   } {
      if {$started} then {
	  click-refresh
	  bring-to-front [self frame-name]
      }
   }
}

loadsource mtsortcommand
