#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soo-Disc.tcl (sort output disc handler setup)   #
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	19/10/04					#
#									#
#########################################################################

set frametitle "MTsort Output Disc Handler Control"
set currentvolume "DISC"
set currentstart 0
set currentend 0

set z [catch {set env(os)} ostype]
if {$z} {set ostype xxx}
if {$ostype == "SunOS"} {
  set devlist {"Not_Active" "Disc"}
  } {
  set devlist {"Not_Active" "Disc"}
  }

set filechoice { \
	"On Output file full: Use next runfile, No-increment" \
	"On Output file full: Use input filename, No-increment" \
	"On Output file full: Use next runfile, Increment filename" \
	"On Output file full: Use input filename, Increment filename" \
	"On Input file change: Use next runfile, No-increment" \
	"On Input file change: Use input filename, No-increment" \
	"On Input file change: Use next runfile, Increment filename" \
        "On Input file change: Use input filename, Increment filename" }


set maxfilesize 0

#------------------------------------------------------------------------------

proc click arg {
   global debug chan stream
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
    clear-footer
    switch -glob $w {

      ADD                    {click-add}
      REPLACE                {click-replace}
      REMOVE                 {click-remove}
      RESETFILE              {click-resetfile}
      SAVE                   {click-save}

      SORT-RUN-FRAME         {click-sort-run-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      RUNLIST                {click-runlist $c $v}

      DEBUG                  {click-debug $v}
      READRUNFILES           {click-reget-runfiles $stream}

      FILENAME-OPTION        {click-filename-option $v}
      FILE                   {click-file $v}
      STARTBLOCK             {click-startblock $v}
      ENDBLOCK               {click-endblock $v}
      MAXFILE                {click-maxfile $v}

      OUTPUT-HANDLER          {click-output-handler $v}

      STREAM                 {click-stream $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug stream
   global env output_style filechoice maxfilesize
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings
   global runfilelist currentstart currentend

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 12c
  set vx2 19c
  set vx3 300
  set vx4 380
  set twid1 15
  set twid2 45

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button ADD :label "Add" :w 2
      :button REPLACE :label "Replace" :w 5
      :button REMOVE :label "Remove" :w 5
      :button RESETFILE :label "ResetLine" :w 6
      :button SAVE :label "Save" :w 2
      :button SORT-RUN-FRAME :label "Sort Run Frame" :x $vx3 :w 12
      :button MYHELP :label "Help"
      :button REFRESH :label "Refresh"
   :panel RUNFILES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :message LIST-HEADER :label "Output Runfiles" :bold T :x 280
      :list RUNLIST :rows 10 :list-width 500 :x 0 :exclusive T \
             :strings $runfilelist :see 0
     :next-row T
      :choice-stack FILENAME-OPTION :label "Output style:" \
       :strings $filechoice :value $output_style
     :next-row T
      :text FILE :label "File:" :w $twid2 :valuex $vx1
      :text MAXFILE :label "MAX (Gbytes):" :w 3
     :next-row T
      :text STARTBLOCK :label "Start block:" :valuex $vx1
      :choice-stack DEBUG :label "Debug" :strings $debugmenustrings :x $vx4
     :next-row T
      :text ENDBLOCK :label "End block:" :valuex $vx1
      :button READRUNFILES :label "ReRead from Setup" :x $vx4
  :panel OPTIONS  :layout L :padx 10 :pady 10
     :layout H; :col-gap 10
     :message OUTPUT-HANDLER :label "Output handler:    $handlerloaded" :bold T
     :next-row T
     :message STREAM :label "Stream $stream" :bold T
   :panel INFO  :layout L :padx 10 :pady 10
      :layout V; :col-gap 10
      :text OUTPUT-HANDLER-STATUS :label "Status:" :value $status :bold T \
         :w $twid1 :valuex $vx2
      [:path OUTPUT-HANDLER-STATUS].entry configure -bd 0
      :text STATISTICS-TOTAL :label "STATISTICS    total:" :value 0 :bold T \
         :w $twid1 :valuex $vx2
      [:path STATISTICS-TOTAL].entry configure -bd 0
      :text STATISTICS-CURRENT :label "current:" :value 0 :bold T \
         :w $twid1 :valuex $vx2
      [:path STATISTICS-CURRENT].entry configure -bd 0
   :show T

   :modify STARTBLOCK :value $currentstart
   :modify ENDBLOCK :value $currentend
   :modify MAXFILE :value $maxfilesize
}

proc paint-handler-status {} {
   global status basedir
   set z [mtsort $basedir "o state" status 1]
   :modify OUTPUT-HANDLER-STATUS :value $status
   }

proc repaint {} {
   global basedir stream handlerselected handlerloaded
   global devlist deviceselected
   global tracelist tracelevel

   :modify STREAM :label "Stream $stream"

   check-handler
   :modify OUTPUT-HANDLER :label "Output handler:    $handlerloaded"
   paint-handler-status

   set m ""
   set z [mtsort $basedir "o stats $stream" m 1]
   if {$z} then { set statistics1 "Unknown q Unknown q"} \
   else { set statistics1 [string range $m 27 end] }
   set statistics2 "0 block 0 current"

   :modify STATISTICS-TOTAL :value [first $statistics1]
   :modify STATISTICS-CURRENT :value [third $statistics1]
   :modify DEBUG :value $tracelevel

   paint-runfiles

#   get-filename-option
#   paint-filename-option
   paint-maxfile-option
   }

proc read-runinfo {} {
    global basedir stream runfilename runfilelist
    global currentvolume currentfile currentstart currentend
    set runfilelist {}
    set i 0
    set z 0
    set m ""
    while {$z==0} {
	set i [expr $i + 1]
	set z [mtsort $basedir "o runinfo $stream $i ?" m 1]
	if {$z==0} {
	    if {[string range $m 0 0] != " "} {lappend runfilelist [format-runfile $m]}
	}
    }
    
    paint-runfiles
}

proc paint-runfiles {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
    :modify RUNLIST :empty T :strings $runfilelist :select $runfileselected :see $runfileselected
    if {$runfilelist != {}} {
	set runfile [lindex $runfilelist $runfileselected]
	if {$runfile == ""} {break}
	set currentvolume "DISC"
	set currentfile [second $runfile]
	set currentstart [third $runfile]
	set currentend [fourth $runfile]
	:modify FILE :value $currentfile
    }
    :modify STARTBLOCK :value $currentstart
    :modify ENDBLOCK :value $currentend
}

proc click-stream {e} {
   global basedir stream
   global deviceselected devlist
   frame-busy
   set deviceselected $e
   set d [lindex $devlist $e]
    set m ""
   set z [mtsort $basedir "o device $stream $d" m 1]
   if {$z!=0} then {set-footer $m; set deviceselected -1} \
   else {
      clear-footer
      }
   repaint
   frame-idle
   }

proc click-maxfile s {
   global maxfilesize basedir
   set maxfilesize  $s
   set m ""
   set z [mtsort $basedir "o maxfile $maxfilesize" m 1]
   if {$z!=0} then {set-footer $m} \
   else {clear-footer}
   repaint
   }

proc paint-maxfile-option {} {
   global maxfilesize
   :modify MAXFILE :value $maxfilesize
}


proc check-file f {
    if {[string index $f 0] == "/"} {return 1} {
	set-footer "Full pathname required"
	return 0
    }
}


proc click-file f {
   global currentfile
# may not want tolower: may want asis for disc file
    if {[check-file $f]} {set currentfile  $f} {
	if {![check-file $currentfile]} {set currentfile ""}
	:modify FILE :value $currentfile
    }
}

proc click-add {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if { $currentvolume == ""} {return}
   if { $currentfile == ""} {return}
   if {![check-file $currentfile]} {return}
   frame-busy
   set l "$currentvolume $currentfile $currentstart $currentend"
   lappend runfilelist [format-runfile $l]
   set runfileselected [expr [llength $runfilelist] - 1]
   paint-runfiles
   frame-idle
   }

proc click-replace {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if { $currentvolume == ""} {return}
   if { $currentfile == ""} {return}
   if {![check-file $currentfile]} {return}
   frame-busy
   set l "$currentvolume $currentfile $currentstart $currentend"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-resetfile {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
   set currentstart 0
   set currentend 0
   set l "$currentvolume $currentfile $currentstart $currentend"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-remove {} {
   global runfilelist runfileselected
   frame-busy
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected]
   set runfileselected [expr $runfileselected - 1]
   if { $runfileselected < 1 } then {set runfileselected 0}
   paint-runfiles
   frame-idle
   }

proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   paint-runfiles
#   get-filename-option
   paint-filename-option
   repaint
   frame-idle
   }

proc click-runlist {c f} {
   global debug
   global runfile runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if {$debug} then {puts stderr "click-list: $c $f"}
   clear-footer

   case $c in {
      :list-select {
         set runfile $f
         set currentvolume "DISC"
         set currentfile [second $runfile]
         set currentstart [third $runfile]
         set currentend [fourth $runfile]
         set runfileselected [lsearch $runfilelist $runfile]
        }
     :list-deselect {
        set runfile ""
         set runfileselected 0
        }
     }          
   if {$runfileselected < 1} then { set runfileselected 0 }
   paint-runfiles
   }

proc init-values {} {
   global debug output_style maxfilesize
   global basedir stream tracelevel tracelist
   global devlist deviceselected

   read-handler-directory

    set m ""
   set z [mtsort $basedir "o trace ?" m 1]
   set tracelevel $m
   if {$debug} then {puts "soo init: trace $z $m"}

   set z [mtsort $basedir "o device $stream ?" m 1]
   if {$z==0 && $m!=""} then { set deviceselected [lsearch $devlist $m] } \
                        else { set deviceselected 0 }
   if {$debug} then {puts "soo init: device$stream $z $m"}

# Set output style

    set z [catch {get-eg-option mtsort-output-style-stream[set stream]} m]
    if {!$z} {set output_style $m}
    set m ""
    set z [mtsort $basedir "o style $stream ?" m 1]
    if {$z==0} {set output_style $m} {
	set m ""
	set z [mtsort $basedir "o style $stream $output_style" m 1]
    }
    edit-defaults mtsort-output-style[set stream] "set-eg-option mtsort-output-style[set stream] $output_style"

# Set max file size

   set maxfilesize 2 
   set z [catch {get-eg-option mtsort-output-maxfilesize[set stream]} m]
    if {!$z} {set maxfilesize $m}
    set m ""
    set z [mtsort $basedir "o maxfile ?" m 1]
    if {$z==0} {set maxfilesize $m} {
	set m ""
	set z [mtsort $basedir "o maxfile $maxfilesize" m 1]
    }
    edit-defaults mtsort-output-maxfilesize[set stream] "set-eg-option mtsort-output-maxfilesize[set stream] $maxfilesize"

   return 0
   }


proc start-handler args {
   global started output_style maxfilesize
   if {![init-values]} then { 
       make-frame
       update idletasks
       click-refresh
       set started 1
   }
}
