#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soo-tape.tcl (sort output null handler setup)   #
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	25/4/97						#
#									#
#########################################################################

set frametitle "MTsort Output Null Handler Control"
set currentvolume "NULL"
set currentfile "NULL"

set z [catch {set env(os)} ostype]
if {$z} {set ostype xxx}
if {$ostype == "SunOS"} {
  set devlist {"Not_Active" "Null"}
  } {
  set devlist {"Not_Active" "Null"}
  }

#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      ADD                    {click-add}
      REPLACE                {click-replace}
      REMOVE                 {click-remove}
      SAVE                   {click-save}

      SORT-RUN-FRAME         {click-sort-run-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      RUNLIST                {click-runlist $c $v}

      DEBUG                  {click-debug $v}

      STREAM                 {click-stream $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug
   global env
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings
   global runfilelist

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 12c
  set vx2 19c
  set vx3 300
  set twid1 15
  set twid2 45

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :choice-stack DEBUG :label "Debug" :strings $debugmenustrings
      :button SORT-RUN-FRAME :label "Sort Run Frame" :x $vx3
      :button MYHELP :label "Help"
      :button REFRESH :label "Refresh"
  :panel OPTIONS  :layout L :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 10
     :message OUTPUT-HANDLER :label "Output handler:    $handlerloaded" :bold T
     :next-row T
     :message STREAM :label "Stream 1" :bold T
   :panel INFO  :layout L :padx 10 :pady 10
      :layout V; :row-gap 20; :col-gap 10
      :text OUTPUT-HANDLER-STATUS :label "Status:" :value $status :bold T \
         :w $twid1 :valuex $vx2
      [:path OUTPUT-HANDLER-STATUS].entry configure -bd 0
      :text STATISTICS-TOTAL :label "STATISTICS    total:" :value 0 :bold T \
         :w $twid1 :valuex $vx2
      [:path STATISTICS-TOTAL].entry configure -bd 0
      :text STATISTICS-CURRENT :label "current:" :value 0 :bold T \
         :w $twid1 :valuex $vx2
      [:path STATISTICS-CURRENT].entry configure -bd 0
   :show T
}

proc paint-handler-status {} {
   global status basedir
   set z [mtsort $basedir "o state" status 1]
   :modify OUTPUT-HANDLER-STATUS :value $status
   }

proc repaint {} {
   global basedir stream handlerselected handlerloaded
   global devlist deviceselected
   global tracelist tracelevel

   check-handler
   :modify OUTPUT-HANDLER :label "Output handler:    $handlerloaded"
   paint-handler-status

    set z [mtsort $basedir "o stream 1" m 1]
   set z [mtsort $basedir "o stats $stream" m 1]
   if {$z} then { set statistics1 "Unknown q Unknown q"} \
   else { set statistics1 [string range $m 27 end] }
   set statistics2 "0 block 0 current"

   :modify STATISTICS-TOTAL :value [first $statistics1]
   :modify STATISTICS-CURRENT :value [third $statistics1]
   :modify DEBUG :value $tracelevel

   paint-runfiles
   }

proc set-runinfo {} {
   global runfilename runfilelist
   global currentvolume currentfile currentstart currentend

    set l "$currentvolume $currentfile $currentstart $currentend"
    lappend runfilelist [format-runfile $l]
    set runfileselected [expr [llength $runfilelist] - 1]
    click-save
   }

proc read-runinfo {} {
   global basedir stream runfilename runfilelist
   set runfilelist {}
   set i 0
   set z 0
    set m ""
   while {$z==0} {
      set i [expr $i + 1]
      set z [mtsort $basedir "o runinfo $stream $i ?" m 1]
      if {$z==0} then {
         lappend runfilelist [format-runfile $m]
         }
      }

   if {$i == 1} {set-runinfo}

   paint-runfiles
   }

proc paint-runfiles {} {
   }
                 
proc click-stream e {
   global basedir stream
   global deviceselected devlist
   frame-busy
   set deviceselected $e
   set d [lindex $devlist $e]
    set m ""
   set z [mtsort $basedir "o device $stream $d" m 1]
   if {$z!=0} then {set-footer $m; set deviceselected -1} \
   else {
      clear-footer
      }
   repaint
   frame-idle
   }

proc click-add {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
    if { $currentvolume == ""} return
    if { $currentfile == ""} return
    set currentstart 0
    set currentend 0
   set l "$currentvolume $currentfile $currentstart $currentend"
   lappend runfilelist [format-runfile $l]
   set runfileselected [expr [llength $runfilelist] - 1]
   paint-runfiles
   frame-idle
   }

proc click-replace {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
   set l "$currentvolume $currentfile $currentstart $currentend"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-remove {} {
   global runfilelist runfileselected
   frame-busy
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected]
   set runfileselected [expr $runfileselected - 1]
   if { $runfileselected < 1 } then {set runfileselected 0}
   paint-runfiles
   frame-idle
   }

proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   paint-runfiles
   repaint
   frame-idle
   }

proc click-runlist {c f} {
   global debug
   global runfile runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if {$debug} then {puts stderr "click-list: $c $f"}
   clear-footer

   case $c in {
      :list-select {
         set runfile $f
         set currentvolume [first $runfile]
         set currentfile [second $runfile]
         set currentstart [third $runfile]
         set currentend [fourth $runfile]
         set runfileselected [lsearch $runfilelist $runfile]
        }
     :list-deselect {
        set runfile ""
         set runfileselected 0
        }
     }          
   if {$runfileselected < 1} then { set runfileselected 0 }
   paint-runfiles
   }

proc init-values {} {
   global debug
   global basedir stream tracelevel tracelist
   global devlist deviceselected

   read-handler-directory

    set m ""
   set z [mtsort $basedir "o trace ?" m 1]
   set tracelevel $m
   if {$debug} then {puts "soo init: trace $z $m"}

   set z [mtsort $basedir "o device $stream ?" m 1]
   if {$z==0 && $m!=""} then { set deviceselected [lsearch $devlist $m] } \
                        else { set deviceselected 0 }
   if {$debug} then {puts "soo init: device$stream $z $m"}

   return 0
   }


proc start-handler args {
   global started
   if {![init-values]} then { 
      make-frame
      update idletasks
      click-refresh
      set started 1
      }
   }
