# sortpdb.tcl---sort polygon directory browser

# Fri Jan 23 14:08:51 GMT 1998, JAS
# polygon editor MTsort version for editing 2D gates

set debug 0

set started 0

set frame SORTPDB
set frametitle "MTsort 2D Gate Editor"
set icontitle GateEditor

if {[info exists env(SORTSPECPATH)] && [file exists $env(SORTSPECPATH)]} then {
   set pathnames(0) $env(SORTSPECPATH)
} else {
   set pathnames(0) $env(HOME)
}

set directory $pathnames(0)
set resource ""

set file ""
set savefile ""
set dirstrings {"Sort matrices" "Sort *DATA gates"}
set dirchoice 0
set filechoices(0) {}
set filechoices(1) {}

set sortfile ""
set stardatafile ""

loadsource sb.common 

set polygonwidgets {
    {{"load polygon array"}        {self}    {click-load}}
    {{"save polygon array"}        {self}    {click-save}}
    {{"select next polygon"}       {sd}      {click-next-polygon}}
    {{"create polygon from tags"}  {sd}      {click-add-polygon}}
    {{"delete selected polygon"}   {sd}      {click-delete-polygon selected}}
    {{"delete all polygons"}       {sd}      {click-delete-polygon -1}}
    {{"add vertex"}                {sd}      {click-add-vertex}}
    {{"delete vertex"}             {sd}      {click-delete-vertex}}
    {{"project spectrum"}          {sd}      {click-project-spectrum}}
}

set polygonmenu {}
foreach p $polygonwidgets {lappend polygonmenu [first $p]}

proc make-frame {} {
   global env
   global dirstrings sortfile dirchoice polygonmenu

   :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
      :icon-image "@$env(ICONHOME)/sort"  \
      :icon-name [self icon-title]

    :panel P0 :layout V :pady 5 :padx 10
     :layout H
      :choice DIRECTORYCHOICE :label "Directory displayed" \
          :strings $dirstrings :value $dirchoice
      :hskip 100

    :panel P1 :layout V :pady 5 :padx 10
     :layout H
      :text DIRECTORY :label "Directory" :w 50 :shifters T
      :hskip 20
      :button UP; :button DOWN
      :hskip 40
      :button REDISPLAY; :hskip 5; :button MYHELP 

    :panel P2 :layout V :padx 10
      :list DIRECTORYLIST :rows 10 :cols 80 :exclusive T :double-click T

    :panel P3 :layout V :pady 5 :padx 10
     :layout H; :row-gap 10; :col-gap 5
      :button SHOW :label "Show matrix"
      :hskip 60
      :button LOAD :label "Load gates";  :button SAVE :label "Save gates"
      :hskip 20; :col-gap 0
      :button SAVE-AS :label "Save As"
      :text FILE :label "" :w 30
      :menubutton POLYEDIT :label "Edit gates" :menu $polygonmenu

    :panel P5 :layout V :pady 5 :padx 10
     :layout H; :row-gap 10; :col-gap 5
      :button MAKEPOLYARRAYS :w 50 \
             :label "Generate polyarray files from new sortfile"
      :text SORTFILE :label "Sortfile:" :w 40 :value $sortfile :valuex 80

    :panel P6 :layout V :pady 5 :padx 10
     :layout H; :row-gap 10; :col-gap 5
      :button STARDATAFORMAT :w 50 \
             :label "Generate MTsort *DATA statements from polyarrays"
      :text STARDATAFILE :label "Filename:" :w 40 :valuex 80

    :show T
}

proc click-polyedit {command} {
   global debug polygonwidgets
   if {$debug} {puts stdout "click-polyedit: command=$command"}
   foreach widget $polygonwidgets {
      if {[string match [first $widget] \"$command\"]} {
	  set interps [second $widget]
	  set action [third $widget]
          if {$debug} {puts stdout "interps=$interps, action = $action"}
	  if {$interps=="self"} {
	      set z [$action]
	  } else {
	      set z [inform $interps $action]
	  }
	  if {$debug} {puts stdout "z=$z"}
	  if {$z!=""} {midas-warning "$z"}
	  return
      }
   }
}

proc double-click arg {
   global debug
   if {$debug} then {puts stderr "double-click with argument \{$arg\}"}
   set w [lindex $arg 2]

   switch -glob $w {
        DIRECTORYLIST        {click-down-or-load}
	default     {puts stdout $arg}
   }
}

proc click-directorychoice f {
   global debug
   global directory pathnames dirchoice filechoices selection
   if {$debug} {puts stdout "f=$f"}
   if {$dirchoice != $f} {
      if {$debug} {puts stdout "pathnames($dirchoice)=$pathnames($dirchoice) directory=$directory"}
      set pathnames($dirchoice) $directory
      set filechoices($dirchoice) $selection
   }
   set dirchoice $f
   set directory $pathnames($f)
   set selection $filechoices($f)
#   if {$debug} {puts stdout "llength selection=[llength $selection]"}
   set-button-states $f
   click-redisplay
   if {[llength $selection] == 1 && [first $selection]>0} {
      set choice [first $selection]
      :modify DIRECTORYLIST :select $choice
      :modify DIRECTORYLIST :see $choice
   }
}

proc set-button-states b {
   if {!$b} {set state1 F; set state2 T} {set state1 T; set state2 F}
   :modify SHOW :inactive $state1
   :modify LOAD :inactive $state2
   :modify SAVE :inactive $state2
   :modify SAVE-AS :inactive $state2
   :modify FILE :inactive $state2
   :modify MAKEPOLYARRAYS :inactive $state2
   :modify STARDATAFORMAT :inactive $state2
}

proc click-file f {
   global savefile
   set savefile $f
}

proc click-show {} {
   global frame
   global selection directorylist
#   upvar #0 $frame-selection selection
#   upvar #0 $frame-dirlist dirlist

   clear-footer
   if {[llength $selection] == 1} then {
     set target [lindex $directorylist [first $selection]]
     if {[string first d $target] == 0} then {
        set-footer "You cannot show a directory - Please select a spectrum"
        return
     }
   }

   set-busy
   set prefix [get-directory-slash]
   set spectrumname $prefix[last $target]
   set z [catch {inform sd show $spectrumname} m]
   if {$z!=0} then {
       set-footer "$m"
   } else {
       click-directorychoice 1
       :modify DIRECTORYCHOICE :value 1
       bring-to-front $frame
       midas-report "Select polygon to display from list of *DATA gates"
   }
   clear-busy
}

proc get-resource {} {
    global resource
    if {$resource==""} then {return ""} else {return "/$resource"}
}

proc get-directory {} {
    global resource directory
    if {$directory==""} then {return "[get-resource]"} \
       else {if {$resource==""} then {return "$directory"} else {return "[get-resource]/$directory"}}
}

proc get-directory-slash {} {
    set d [get-directory]
    if {$d==""} then {return "$d"} else {return "$d/"}
}

proc match-selection {} {
   global selection dirchoice
   global choosestring
   global sortfilestrings
   if {$dirchoice == 0} then {
      set selection [lsearch $sortfilestrings $sortfile]
      }  \
   else  {
      set selection [lsearch $sortfilestrings ]
      }
   if {$nthsortfile>=0} then {
      update-sort-file-name
      set choosestring F
      } \
   else {set choosestring T}
   }

proc click-load {} {
   global selection directorylist directory file
   if {[llength $selection] == 0} then {
      set-footer "no polyarray file selected!!!"
      return
   }
   if {[llength $selection] > 1} then {
      set-footer "multiple selections!!!"
      return
   }
   set target [lindex $directorylist $selection]
   if {[string index $target 0] == "d"} then {
      set-footer "can't load a directory!!!"
      return
   }
   set file [last $target]
   inform sd load-polyarray-file $directory $file
}

proc click-down-or-load {} {
   global selection directorylist directory file dirchoice pathnames

   set pathnames($dirchoice) $directory

   if {[llength $selection] == 0} then {
      set-footer "no polyarray file selected!!!"
      return
   }
   if {[llength $selection] > 1} then {
      set-footer "multiple selections!!!"
      return
   }
   set target [lindex $directorylist $selection]
   set file [last $target]
   if {[string index $target 0] != "d"} then {
      if {$dirchoice==0} then {
         click-show
         return
      } else {
         inform sd load-polyarray-file $directory $file
         return
      }
   }
   if {"$file" == "."} then {click-redisplay; return}
   if {"$file" == ".."} then {click-up; return}
   if {$directory == "/"} then {set directory ""}
   append directory / $file
   click-redisplay
   set selection {}
}

proc click-save {} {
   global directory file
   if {"$file" == ""} then {
      set-footer "you haven't loaded a polyarray file yet!!!"
      return
   }
   set z [inform sd save-polyarray-file $directory $file sortpdb]
   if {$z==""} {
      set-footer "Polyarray saved as [get-directory-slash]$file"
   } else {
      set-footer $z
   }
}

proc call-back {interp proc result} {inform $interp "$proc [transparent [$result]]"}

proc click-save-as {} {
   global directory savefile
   if {"$savefile" == ""} then {
      set-footer "you haven't given a new file name!!!"
      return
   }
   set z [inform sd save-polyarray-file $directory $savefile sortpdb]
   if {$z==""} {
      set-footer "Polyarray saved as [get-directory-slash]$savefile"
   } else {
      set-footer $z
   }
}

proc click-makepolyarrays {} {
   frame-busy
   do-makepolyarrays
   click-redisplay
   frame-idle
}

proc do-makepolyarrays {} {
   global debug
   global env sortfile pathnames polydir
   if {$sortfile == ""} {
      display-midas-error-report self "" \
              "You haven't specified a sortfile to read from"
   } else {
      set-footer "Making polyarrays from sortfile $sortfile..."
   }

# Temporary directory for 2D gate files.

   set polydir [file join $env(MIDASTempDir) tcl[pid]-sortmaps]
   if {[file exists $polydir]} then {
      set polynames ""
      catch {set polynames [glob $polydir/*]}
      if {$polynames != ""} {
         foreach poly $polynames {
            set z [catch {:file delete $poly} m]
         }
      }
   } else {
      set z [catch {:file mkdir $polydir} m]
      if {$z != 0} then {
	 midas-warning \
		 "Unable to create directory $polydir for gate files\n$m"
	 return
      }
   }
   cd $polydir

# Convert 2D gates from sortfile into polygon editor format gates.
   set z [catch {exec MTstardata < $sortfile} m]
   if {$z != 0} {midas-warning $m} else {clear-footer}

   set pathnames(1) $polydir
}

proc click-sortfile f {
   global debug sortfile
   if {$debug} {puts stdout "Setting sortfile to $f"}
   set sortfile $f
}

proc click-stardataformat {} {
   global debug env stardatafile pathnames outlist
   frame-busy
   if {$stardatafile == ""} {
      display-midas-error-report self "" \
              "You haven't specified a filename for the *DATA statements"
      frame-idle
   } else {
      set-footer "Generating *DATA statements from polyarrays..."
      set filelist {}
      set filelist [read-directory-names $pathnames(1)]
      set prefix [get-directory-slash]
      set sortstats ""
      foreach f $filelist {
         if {$debug} {puts stdout "processing file $f"}
	 set polyformat [read-file $prefix$f]
	 set sortformat [convert-format $f $polyformat]
	 if {$debug} {puts stdout "sortformat = /$sortformat/"}
	 append sortstats $sortformat
      }
      if {[string first "/" $stardatafile] == -1} {
         set stardatafile $env(HOME)/$stardatafile
      }
      write-file $stardatafile $sortstats
      set-footer "*DATA statements written to $stardatafile"
# Ask user to confirm deletion of map files following conversion to sort format

   }
   frame-idle
}

proc click-stardatafile f {
   global debug stardatafile
   if {$debug} {puts stdout "Setting *DATA filename to $f"}
   set stardatafile $f
}

#------------------------------------------------------------------------------

proc make-vertices p {
    global gallery
    set vs {}
    :spectrum $gallery gate select [poly-number $p]
    set-poly-xys $p [:spectrum $gallery gate get2d]
    set n [poly-order $p]
    set xys [poly-xys $p]
    for {set i 0} {$i<$n} {incr i 2} {
	lappend vs [list [lindex $xys $i] [lindex $xys [expr $i + 1]]]
    }
    return $vs
}

proc read-file fn {
   set f [open $fn r]
   set polys [read $f]
   close $f
   return $polys
}

proc write-file {fn polys} {
   set f [open $fn w]
   puts $f $polys
   close $f
}

proc convert-format {name polys} {
   global debug
   set gatedata ""
   set xmax 0
   set ymax 0
   foreach poly $polys {
      if {$debug} {puts stdout "poly = /$poly/"}
      set gate ""
      foreach xy $poly {
         if {![string length $gate]} {set gate "("} {append gate "  "}
	 append gate $xy
	 set xmax [max [first $xy] $xmax]
	 set ymax [max [second $xy] $ymax]
      }
      if {[string length $gate]} {append gate ")\n"}
      append gatedata $gate
   }
   incr xmax
   incr ymax
   set gateheader [format "GATES 2D $name\[$xmax,$ymax\]\n"]
   return $gateheader$gatedata
}

proc max {a b} {
   if {$a > $b} {return $a} {return $b}
}

proc start {file args} {
   global env started directory pathnames sortfile dirchoice polydir
    if {$started == 0} then {
      make-frame
      set started 1
      midas-report "First select matrix on which to overlay polygonal gates"
   } else {open-frame}

   if { ![info exists env(MIDASTempDir)] } {
      midas-warning "Temporary file directory $env(MIDASTempDir) does not exist\n\
      You can set the environment variable MIDASTempDir to define an alternative location"
      return 0
   }
   set polydir [file join $env(MIDASTempDir) tcl[pid]-sortmaps]
   if {![file exists $polydir]} then {
      set z [catch {:file mkdir $polydir} m]
      if {$z != 0} then {
         midas-warning "" \
		 "Unable to create directory $polydir for gate files\n$m"
         error $m
      }
   }
   set pathnames(1) $polydir
   set sortfile $file
   :modify SORTFILE :value $sortfile
   do-makepolyarrays
   click-directorychoice 0
   :modify DIRECTORYCHOICE :value 0
   set-button-states $dirchoice
   click-redisplay
}
