#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		sos-Sym4dConfig.tcl		                #
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	9/4/97						#
#									#
#########################################################################

set frametitle "MTsort Setup Sym4d Config"
set icontitle "Sym4dConfig"

set list0size 2048
set list0state ""
set list1size 2048
set list1state ""

set array0size 2048
set array0state ""
set array1size 2048
set array1state ""

set range 1024
set listcycle 0
set postcycle 0

set header "          filemax size (MB)         status"

#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      SAVE                   {click-save}

      SORT-SETUP-FRAME       {click-sort-setup-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      LIST0                  {click-list0size $v}
      LIST1                  {click-list1size $v}
      ARRAY0                 {click-array0size $v}
      ARRAY1                 {click-array1size $v}

#      LIST0STATUS            {click-list0state $v}
#      LIST1STATUS            {click-list1state $v}
#      ARRAY0STATUS           {click-array0state $v}
#      ARRAY1STATUS           {click-array1state $v}
#      RANGE                  {click-range $v}
#      LISTCYCLE              {click-listcycle $v}
#      POSTCYCLE              {click-postcycle $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug
   global env header

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}

  set vx1 8c
  set vx2 20c
  set vx3 200
  set twid1  5
  set twid2 10
  set twid3 30

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button SAVE :label "Save"
      :button SORT-SETUP-FRAME :label "Sort Setup Frame" :x $vx3
      :button MYHELP :label "Help"
      :button REFRESH :label "Refresh"
   :panel RUNFILES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :message HEADER :label $header :bold T
     :next-row T
      :text LIST0 :label "list0" :w $twid2 :valuex $vx1
      :message LIST0STATUS :label UNKNOWN :bold T
      :text LISTCYCLE :label "List write cycle:" :w $twid1 :valuex $vx2
     :next-row T
      :text LIST1 :label "list1" :w $twid2 :valuex $vx1
      :message LIST1STATUS :label UNKNOWN :bold T
     :next-row T
      :text ARRAY0 :label "array0" :w $twid2 :valuex $vx1
      :message ARRAY0STATUS :label UNKNOWN :bold T
      :text POSTCYCLE :label "Array write cycle:" :w $twid1 :valuex $vx2
     :next-row T
      :text ARRAY1 :label "array1" :w $twid2 :valuex $vx1
      :message ARRAY1STATUS :label UNKNOWN :bold T
     :next-row T
      :text RANGE :label "range" :w $twid2 :valuex $vx1

   :show T

    bind all <Control-Alt-KeyPress-Delete> {click-quit}
}

proc repaint {} {
   paint-runfiles
   }

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-midas-error-report self "" $m
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc read-runinfo {} {
   global basedir runfilename 
   global list0size list0state list1size list1state
   global array0size array0state array1size array1state
    global range listcycle postcycle

    set i 0
    set z [read-file $runfilename]
    if {$z!=""} then {
	foreach l $z {
	    if {[first $l] == "list0"} {
		set list0size [second $l]
		set list0state [third $l]
	    }
	    if {[first $l] == "list1"} {
		set list1size [second $l]
		set list1state [third $l]
	    }
	    if {[first $l] == "array0"} {
		set array0size [second $l]
		set array0state [third $l]
	    }
	    if {[first $l] == "array1"} {
		set array1size [second $l]
		set array1state [third $l]
	    }
	    if {[first $l] == "range"} {set range [second $l]}
	    if {[first $l] == "list_cycle"} {set listcycle [second $l]}
	    if {[first $l] == "post_cycle"} {set postcycle [second $l]}
	}
    }

    paint-runfiles
}


proc paint-runfiles {} {
   global list0size list0state list1size list1state
   global array0size array0state array1size array1state
    global range listcycle postcycle

   :modify LIST0 :value $list0size
   :modify LIST1 :value $list1size
   :modify ARRAY0 :value $array0size
   :modify ARRAY1 :value $array1size
   :modify LIST0STATUS :label $list0state
   :modify LIST1STATUS :label $list1state
   :modify ARRAY0STATUS :label $array0state
   :modify ARRAY1STATUS :label $array1state
   :modify RANGE :value $range
   :modify LISTCYCLE :value $listcycle
   :modify POSTCYCLE :value $postcycle

   }
                 
proc click-list0size f {
   global list0size
   set list0size  $f
   }

proc click-list1size f {
   global list1size
   set list1size  $f
   }

proc click-array0size f {
   global array0size
   set array0size  $f
   }

proc click-array1size f {
   global array1size
   set array1size  $f
   }

proc click-save {} {
   global runfilename
   global list0size list0state list1size list1state
   global array0size array0state array1size array1state
   global range listcycle postcycle

   frame-busy

    set w ""
    set l "list0  $list0size  $list0state"
    lappend w $l
    set l "list1  $list1size  $list1state"
    lappend w $l
    set l "array0  $array0size  $array0state"
    lappend w $l
    set l "array1  $array1size  $array1state"
    lappend w $l
    set l "range   $range"
    lappend w $l
    set l "list_cycle  $listcycle"
    lappend w $l
    set l "post_cycle  $postcycle"
    lappend w $l

    set z [write-file $runfilename $w]
    if {$z!=0} then {
         display-midas-error-report self "" "Error saving runfiles:\n" $m
         frame-idle
         return
         }
   frame-idle
   }

proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   repaint
   frame-idle
   }

proc init-values {} {
   global debug env
   global basedir 
   global runfilename

   set runfilename $basedir/.sym4d_status

   if {![file exists $runfilename]} then {
      set f [set env(MTSORTBASE)]/config/sym4d_status
      set z [catch {exec cp $f $runfilename} m]
      if {$z} { set-footer "Sym4d config file read failed: $m"}
      }

   return 0
   }


proc start-handler args {
   global started
   if {![init-values]} then { 
      make-frame
      update idletasks
      click-refresh
      set started 1
      }
   }
