# -----------------------------------------------------------------------------
# File stats.tcl
#
# Source code for EVENT BUILDER WINDOW.
#

set debug 0
set frameOpen 0
set framename .stats
set frametitle "MTsort Statistics"
set icontitle "MTStats"

#set ebfont "10x20"
set ebfont "9x15bold"
set mode counts

set stats_names {
"Input Blocks"
"Event Blocks"
"Events Found"
"No Event Header"
"No Event End"
"No Block End"
"Bad Data Format"
"Duplicate Data"
"Output1 Blocks"
"Output1 Events"
"Output2 Blocks"
"Output2 Events"
"Output3 Blocks"
"Output3 Events"
"Output4 Blocks"
"Output4 Events"
}

proc readregs {stats} {
    global basedir
    set z [mtsort $basedir "e stats" m 1]
    if {$z} {return 0}

    set i [string first : $m]
    incr i 2
    set s [string range $m $i end]

    set i [string first : $s]
    set s [string replace $s $i [incr i]]

    set s [split $s]
    set l [llength $s]
    set r {}
    for {set i 0} {$i < $l} {incr i 2} {
	lappend r [list [lindex $s $i] [lindex $s [expr $i +1]]]
    }
    return $r
}

proc =display {rows cols } {
    global ebfont
    set fn [self frame-name]
    text $fn.galley -relief sunken -bd 2 -height $rows -width $cols \
	-font $ebfont -bg #bfe7dc -fg black
    pack $fn.galley -side top -pady 5 -padx 10
}

proc make-frame {} {
    global stats_rows frametitle modes mode

    :frame [self frame-name] :label $frametitle :show-footer T \
	:icon-name "[self icon-title]" :resize F

    set gapy 5
    :panel P1 :padx 10 :pady 2 :layout T
	set fn [self frame-name]
    =display $stats_rows 46
    :panel P3 :padx 10 :pady 2 :layout T
	:message CMODE-MSG :label "COUNT MODE" :x 20 :y $gapy
        :button CMODE :label $mode :w 12 :x 120 :y $gapy :h 1
	:button UPDATE :label "UPDATE" :x 280 :y 5 :w 15 :h 1
    :show T
}

proc finish args {click-quit}

proc verify-stats {} {
    set statslist [readregs stats.*]
    set err 0
    foreach stat $statslist {
	set value [second $stat]
	if {$value} {
	    set err 1
	    break
	}
    }
    return $err
}

set bgcolour(F)	#eec
set bgcolour(T)	green

proc click {arg} {
    global debug clickwidgets
    if {$debug}  {puts  "click with argument \{$arg\}"}
    set f [lindex $arg 0]
    set c [lindex $arg 1]
    set w [lindex $arg 2]
    set v [lindex $arg 3]
    set x [lindex $arg 4]
    if {$c==":quit"} then {finish; return}
    if {$f=="ER"} then {remove-eg-error-report self; return}

    foreach wid $clickwidgets {
	set cwid [lindex $wid 0]
	if { [regexp $cwid $w] } {
            set func [second $wid]
	    break
 	}
    }
    if { [info exists func] } {
	eval $func
    } else {
	puts $arg
    }
}

set clickwidgets {
    { UPDATE 	"click-update"   }
    { CMODE     "click-mode"    }
}

proc click-clear {} {
    set-footer "Clearing statistics ..."
    clear-stats
    save-old-counts
    click-update
}

proc next-list-value {v vals} {
# Find next element in a list
    set i [lsearch $vals $v]
    set i [expr ($i+1)%[llength $vals]]
    set v [lindex $vals $i]
    return $v
}

set modes { counts rates }
proc update-cmode {} {
    global mode modes
    set wid [:path CMODE]
    set v [$wid cget -text]
    set mode [next-list-value $v $modes]
    $wid configure -text $mode
}

proc click-mode {} {
    global mode modes 
    update-cmode
}

proc click-quit {} {
    delete-frame
}

proc click-update {} {
    global display_mode
    click-update[set display_mode]
}

proc click-update {} {
    global mode 
    :modify [self frame-name] :busy T

    if {$mode == "counts"} {
	save-old-counts
	update-counts
    } else {
	update-rates
    }
    :modify [self frame-name] :busy F
}

proc erase-stats {} {
    global debug
    set wid [self frame-name].galley
    $wid delete 1.0 end
}

set head_fmt "%-14s %28s\n"

set samplePeriod 5000

proc update-rates {} {
    global x1 x2 samplePeriod
    set-footer "Updating statistics rates ... wait $samplePeriod msec"
    set x1 [readregs stats.*]
    after $samplePeriod \
	{ set x2 [readregs stats.*]; calculate-rates  }
}

proc calculate-rates {} {
    global debug errorCode eb_ldisp head_fmt x1 x2 samplePeriod \
	rates_time rates_time_prev stats_rates stats_rates_prev

    set t $samplePeriod
    set z [catch {exec date} x]
    set stats_rates_prev $stats_rates
    set rates_time_prev ([lindex $rates_time 3])
    if {$z} {
	set err [lindex $errorCode 2]
	set rates_time "Nul Nul  0 00:00:00 Nul Nul "
    } else {
	set rates_time $x
    }
    erase-stats

    set eb_ldisp 1

    put-stats [format $head_fmt\
	"Rates at:" $rates_time ]

    set nrates [llength $x1]
    set stats_rates {}

    for {set irate 0} {$irate < $nrates} {incr irate} {
	set value1 [lindex $x1 $irate]
	set count1 [second $value1]
	set count2 [second [lindex $x2 $irate]]

	if {$count1 > $count2} {
	    set rate 0;			# Uh, negative rate !
	} else {
	    set rate [expr "($count2 - $count1) / ($t / 1000)"]
	}
	set value_prev [lindex $stats_rates_prev $irate]
	set rate_prev ([second $value_prev])
	set rate_name [first $value1]
	lappend stats_rates [list $rate_name $rate]
	put-stats [format "%-16s %12s %14s" [string range $rate_name 0 end] $rate $rate_prev]

    }
    clear-footer
}


proc update-counts {} {
    global stats_values stats_time
    set l [read-stats]
    set val [lindex $l 0]
    if {$val!=-1} {
	set stats_values $val
	set stats_time [lindex $l 1]
	display-counts
    }
}

proc do-unix {cmd} {
    set z [catch {exec $cmd} m]
    if {$z} {
	display-error-report self $m
	set m (ERROR)
    }
    return $m
}
proc read-stats {} {
    set t [do-unix date]
    set x [readregs stats.*]
    return [list $x $t]
}

proc save-old-counts {} {
    global stats_time stats_time_prev stats_values stats_values_prev

    set stats_values_prev $stats_values
    set stats_time_prev ([lindex $stats_time 3])

}

proc display-counts {} {
    global eb_ldisp stats_time stats_time_prev\
        stats_values stats_values_prev head_fmt
    global stats_names

    erase-stats
    set eb_ldisp 0
    put-stats [format $head_fmt "Counts at:"\
      $stats_time ]
    set istat 0

    foreach stat $stats_values {
	set stat_prev [lindex $stats_values_prev $istat]
	set value_prev ([second $stat_prev])
	put-stats [format "%-16s %12s %14s"\
		       [lindex $stats_names $istat] [second $stat] $value_prev]
	incr istat
    }
    clear-footer
}


proc put-stats {s} {
    global fn 
    set wid [self frame-name].galley
    $wid insert end " $s \n"
}

proc init-globals {} {
    global eb_ldisp mode stats_rows rates_time rates_time_prev\
	stats_rates stats_rates_prev stats_time stats_time_prev\
	stats_values stats_values_prev

    set eb_ldisp -1
    set mode counts
    set rates_time ()
    set rates_time_prev ()
    set stats_rates {}
    set stats_rates_prev {}
    set stats_time ()
    set stats_time_prev ()
    set stats_values [readregs stats.*]
    set stats_rows [llength $stats_values]
    incr stats_rows 2
    set stats_values_prev {}
}

proc start {args} {
    global basedir
    set basedir $args
    init-globals
    make-frame
    click-update
}

loadsource mtsortcommand








